/*
Module : DTime+.h
Purpose: Defines the interface for the DTime+ class library
Created: PJN / 05-05-1995
History: PJN / ??-??-???? 1 Initial public Win32 release.
         PJN / ??-??-???? 1. Port to Win16 and DOS.
                          2. Implementation of a comprehensive OS detection routine.
                          3. A number of classes and functions to make code portable across OS platforms.
                          4. Addition of a number of member functions to return a C runtime tm struct version.
                          5. TermDTime added and InitDTimeDll now renamed to InitDTime.
                          6. printf like function made available under Win16.
                          7. A CStringEx class which provides a Format method under Win16 and DOS and a LoadString method under DOS.
         PJN / ??-??-???? 1. translation to German.
         PJN / ??-??-???? 1. Tighter integration with Win32 NLSAPI.
                          2. Provision of a real control panel applet.
                          3. Enhanced data entry routines.
                          4. Numerous bug fixes.
                          5. Removal of OS detection routine to separate package.
                          6. Removal of Pushpin class to separate package.
                          7. Removal of DOS and Win16 code no longer used.
                          8. DTime has dropped support for millisecond fields in the CLTimeOfDay, CLTimeSpan and CLDate classes. You will 
                          need to check code that was used with DTime v1.x to make sure the correct constructors are being called. This functionality 
                          was removed following customer feedback about its usefulness. The extra fields required to support milliseconds also 
                          introduced a number of subtle bugs that were discovered during testing of v2.0.
         PJN / ??-??-???? 1. v2.0 translation to German.
                          2. Addition of extra CDate and CLDate functions to interact with SYSTEMTIME and FILETIME SDK structures.
                          3. Addition of extra CDate, CLTimeOfDay and CLDate functions which return a tm struct.
                          4. Major modification to CLDate::SetTimeFrame function to correctly handle timezones other than GMT correctly.
                          5. Number of other minor tweaks and bug fixes following feedback from customers.
                          6. Fixed a number of anomalies in the help file dating back to v1.1.
                          7. Enhanced the data entry methods DTime supports. It now allows a Julian Day to be entered. Also additional 
                          flags have been added to control how the data entry operates. 
                          8. All the arithmetic operators have been enhanced to handle invalid state of their operands, instead of just 
                          asserting when an invalid operand is found.
         PJN / ??-??-???? 1. Modification to serialize functions to save space on disk.
                          2. Replaced use of AFX_EXT_API and AFX_EXT_CLASS to allow use of DTime in another extension dll.
                          3. Added support for using the DTime source code directly in your application instead of through an Extension DLL.
                          4. Added a DTIME_NO_OLE_SUPPORT macro which removes the dependency of DTime on the MFC OLE dlls. Note that this can 
                             only be used if you include the DTime source code directly in your application.
                          5. Changed the load address of all variants of the DTime dll. This reduces the occurrences of dll collisions at 
                             process load up time.
                          6. Made DTime and its sample programs compatible with VC++ 5.0.
                          7. Added support in DTime for Proleptic Julian and Gregorian Calendars.
                          8. Added support for converting from a CLDate to a COleDateTime.
                          9. Removed the Apply Now buttons from the data entry dialogs.
                          10. Major revamp of the install program to give it a more professional look.
                          11. Major update to the help file. Includes changes suggested by customers such as making its look and feel more 
                          consistent with VC++ books online and Windows 95 Help. Also fixed a number of mistakes which were in the 2.1 version 
                          of the help file.
         PJN / 17-07-1998 1. Updated 2 files for DTime which fixes a problem with usage of DTime in a multithreading application. Basically it 
                          boiled down to code in DTime which was making changes to a global array "MonthLength" from two threads. Now a local 
                          array is used, thus preventing the problem. Just copy the two files into your DTime "SRC" directory over the existing 
                          files of the same name.
         PJN / 14-05-1999 1. Updated 4 files for DTime. This fixes a number of problems DTime was having when compiled with Visual C++ 6, Just copy 
                          the three cpp files into your DTime "SRC" and the one h file into your DTime "Include" directory over the existing 
                          files of the same name. 4 files are included in total (1 file which was modified in v2.21) meaning that you do not need 
                          to apply the v2.21 patch as the v2.22 patch includes all of the changes since the v2.2 main release.
         PJN / 11-04-2000 1. Refresh of the code following the creation of my new site http://www.naughter.com
         PJN / 22-04-2006 1. Following a long term of hibernation, DTime is now back. It is now freeware as opposed to Shareware and has been 
                          renamed to DTime+.
                          2. Updated copyright details.
                          3. Classes are no longer derived from CObject as this was not really necessary.
                          4. Reworked the layout of the directories to ship as a more standard collection of standard C++ modules. Like other code 
                          provided by the author. the code now does not ship prebuilt as an extension dll, instead it is provided as a pure source 
                          code library. You are of course free to include DTime+ in an extension dll if you so desire.
                          5. Removed the timeframe concept from CLDate instance data
                          6. Removed support for ET and DeltaT. If you would like support for this please check out the author's AA+ class 
                          framework at http://www.naughter.com/aa.html.
                          7. Integrating the concept of a Windows locale (LCID) into all functions which return strings. This allows the code to 
                          easily mix and match calendar strings from multiple languages in the one program.
                          8. Because the code now tightly integrates with the Win32 NLS API's, DTime+ now does not need to carry around any 
                          resources of its own.
                          9. Removed all virtual functions from all DTime+ classes
                          10. Removed the need to use friend classes
                          11. Updated the references to where the basic algorithms used by DTime+ are based on. See the comments in the
                          CDate::Set() method.
                          12. Provided updated UI support via the standard Windows date / time controls.
         PJN / 01-05-2005 1. Reworked the CDate and CLDate classes to use templates to specify the calendar type to use (Julian, Gregorian or 
                          Moslem). By making these classes template based, we have dispensed with a bool member variable in each instance, thereby 
                          reducing the per instance memory size of each of these classes. It also means that the serialized size for both of these 
                          classes has shrunk by 1 byte. By also making the classes template based, the code should also have a better chance of being 
                          optimized by the compiler. An example of the changes this implies to client code is as follows:

                          CDate x(1990, CDate::March, 4, CDate::Gregorian) -> DTimePlus::CDate<CGregorianCalendar>(1990, March, 4) and

                          CDate x(1990, CDate::March, 4, CDate::Julian) -> DTimePlus::CDate<CJulianCalendar>(1990, March, 4)

                          2. Because of the move to templates, I had a problem getting CLTimeSpan and CLTimeOfDay to compile because there is a 
                          circular dependence between these two classes. When you make a class template based, using forward class references to
                          resolve these issues now no longer works. Instead the CLTimeOfDay is now also template based which works around the
                          circular reference issue. An example of the change this implies to client code is as follows:

                          CLTimeOfDay y(10, 12, 14) -> DTimePlus::CLTimeOfDay<> y(10, 12, 14);

                          3. Moved all the classes and methods out of the global namespace and into a namespace called "DTimePlus"
                          4. Implemented extensive support for the Moslem Calendar. The algorithms are taken from the book 
                          "Calendrical Calculations, The Millenium Edition" by Edward M. Reingold and Nachum Dershowitz. The 4 most common forms
                          of the Moslem Arithmetic calendar are supported namely the 15th and 16th year leap year rules as well as the Civil Epoch
                          (16 July 622 C.E. O.S) and the Astronomical Epoch (15 July 622 C.E. O.S). These are implemented by the template 
                          classes CMoslemCalendarCivil16, CMoslemCalendarCivil15, CMoslemCalendarAstronomical16 and finally 
                          CMoslemCalendarAstronomical15. For example to create a date which represents Muharram 1, A.H. 1 using the Civil epoch
                          and the 16th year leap rule you would use:

                          DTimePlus::CDate<CMoslemCalendarCivil16> FirstDayOfMoslemEpoch(1, Muharram, 1);

                          The Moslem month's are defined as a standard enum in the DTimePlus namespace just like you can use the Julian / Gregorian
                          month names.
                          5. Optimized the operation of the various Format methods.
         PJN / 20-05-2006 1. CGregorianCalendar now uses functors for leap year calculations
                          2. Moslem classes now uses functors for leap year functionality.
                          3. CJulianCalendar now uses functors for leap year calculations
         PJN / 22-05-2006 1. Fixed a compile issue in VC 2003 with the multiplication operator for
                          CLTimeSpan. The issue was related to the tighter ISO conformance of the
                          compiler. Thanks to "Stilgar" for reporting this issue.
         PJN / 25-05-2006 1. Optimized the CBaseMoslemCalendar::DaysInMonth code.
                          2. Made some Moslem functions inline to aid optimization
                          3. Optimized the code in CJulianGregorianCalendar::DaysInMonth
                          4. Made all the functions in CJulianGregorianCalendar inline
                          5. Optimized the DTimePlus::WeekOfDayModulo and DTimePlus::lfloor functions. This
                          results in a 2 fold increase in the speed of the key algorithms for the Julian
                          and Gregorian calendars.
                          6. Updated the string and enums for the 7th month to be "Tishrei" in the Hebrew calendar. Thanks to Itamar Syn-Hershko for 
                          reporting this issue.
                          7. Updated the string and enums for the first month to be "Nissan" in the Hebrew calendar. Thanks to Itamar Syn-Hershko for 
                          reporting this issue.
                          8. Updated the string and enums for the 8th month to be "Cheshvan" in the Hebrew calendar. Thanks to Itamar Syn-Hershko for 
                          reporting this issue.
                          9. Now includes spaces in the strings returned for AdarI and AdarII in the Hebrew calendar.
                          10. Fixed a typo in the string and enums for the 11th month "Shevat" in the Hebrew calendar. Thanks to Itamar Syn-Hershko 
                          for reporting this issue.
                          11. Made some Hebrew functions inline to aid optimization
         PJN / 27-05-2006 1. Reworked Moslem algorithms to support dates before the Moslem epoch. i.e. a proleptic Moslem calendar
                          2. Updated the month enum names and month strings int the Moslem calendar
                          3. Updated the weekday strings in the Moslem calendar
                          4. Removed the need for the function WeekOfDaysModulo
                          5. Reworked Hebrew algorithms to support dates before the Hebrew epoch i.e.
                          a proleptic Hebrew calendar
                          6. Renamed the LongMarheshvan function to LongCheshvan in the Hebrew calendar.
                          7. Renamed the ShortMarheshvan function to ShortCheshvan in the Hebrew calendar
         PJN / 29-05-2006 1. Further performance optimizations for the core CGregorianCalendar algorithms.
                          2. Updated the month enum name and month strings for "Ramadan"
                          3. Further performance optimizations for the core CJulianCalendar algorithms.
                          4. Removed the now unnecessary defines DTIMEPLUS_EXT_CLASS and DTIMEPLUS_EXT_API. 
                          These are no longer required because DTime+ is template based.
         PJN / 30-05-2006 1. Added support for Rosh HaShanah 2 in the Hebrew calendar
                          2. Renamed CSimhatTorahFunctor to CCSimhatTorahOutsideIsraelFunctor in the Hebrew calendar
                          3. Renamed CPassoverFunctor to CPesachFunctor in the Hebrew calendar
                          4. Renamed CEndingOfPassoverFunctor to CShviiPesachFunctor in the Hebrew calendar
                          5. Added support for Yom Ha'atzmaut in the Hebrew calendar
                          6. Added support for Erev Pesach in the Hebrew calendar
                          7. Added support for Ta'anit Bechorim in the Hebrew calendar
                          8. Added support for Pesach Shenni in the Hebrew calendar
                          9. Added support for Yom Yerushalaiym in the Hebrew calendar
                          10. Renamed to CHoshahaRabbaFunctor to CHoshanahRabahFunctor in the Hebrew calendar
         PJN / 02-07-2006 1. Updated the code to clean compile on VC 2005
                          2. Code now uses new C++ style casts rather than old style C casts where necessary.
         PJN / 13-04-2007 1. Updated copyright details.
                          2. Updated sample apps to clean compile on VC 2005
                          3. Fixed a bug in CDate::Set(const CTime& ctime, ...) and CLDate& CLDate::Set which incorrectly used GetGmtTm instead of 
                          GetLocalTm. Thanks to Martin Copland for reporting this bug.
         PJN / 26-10-2014 1. Updated copyright details
                          2. Updated code to clean compile with /analyze
                          3. CJulianGregorianHolidays::StStephensDays has been renamed to StStephensDay
                          4. Addition of a number of new methods in the CJulianGregorianHolidays class including CanadaThanksgivingDay & 
                          EasterMonday
                          5. Made improvements to the HebrewOccurencesInGregorianYear method
                          6. Reworked the SafeGetLocaleInfo, SafeGetDateFormat & SafeGetTimeFormat to avoid the use of _alloca.
                          7. Fixed a bug in CLTimeOfDay::GetAMPMString related to when PM kicks in.
                          8. Fix arithmetic bugs in a number of the CLDate::Set methods related to the calculation of m_nSeconds. Thanks to 
                          Zhen-dong Chen for reporting these issues.
                          9. Fix arithmetic bugs in CLDate::GetCDate. Thanks to Zhen-dong Chen for reporting these issues.
                          10. Fix arithmetic bugs in CLDate::GetCLTimeOfDay. Thanks to Zhen-dong Chen for reporting these issues.
         PJN / 06-12-2016 1. Updated copyright details
                          2. Removed all usage of CString from the code and replaced with std::[w]string.
                          3. Replaced all occurrences of WORD with uint16_t throughout the code
                          4. Replaced all occurrences of DWORD with uint32_t throughout the code 
                          5. Replaced all occurrences of ASSERT, AFXASSUME, TRACE & VERIFY with non MFC dependent implementations
                          6. Replaced all occurrences of long with int32_t throughout the code
                          7. Renamed the DMYToLong method to "DMYToNum"
                          8. Optimized the elements of the static lookup tables in CJulianCalendar::DMYToNum & CJulianCalendar::GetDate
                          9. Replaced all occurrences of NULL with nullptr throughout the code. This does mean that the classes now require
                          at least VC 2010 if you are going to use a Microsoft compiler.
                          10. Reimplemented CDate::CurrentSystemDate to just use the C runtime.
                          11. Reimplemented CDate::CurrentLocalDate to just use the C runtime.
                          12. Optimized the CDate::Set method which takes a COleDateTime parameter
                          13. Optimized the CLDate::Set method which takes a COleDateTime parameter
                          14. Fixed a bug in the validation code of the CDate::Set method which takes a "WeekOfMonth" parameter.
                          15. Fixed a bug in CJulianGregorianHolidays::EasterMonday where it returned EasterSaturday by mistake
                          16. Fixed all typos of "occurences" throughout code and documentation
                          17. Optimized the CDate::Set method which takes a CTime parameter
                          18. Optimized the CLDate::Set method which takes a CTime parameter
         PJN / 20-12-2016 1. Optimized the CGregorianLeapFunctor::operator() method per the information at
                          https://howardhinnant.github.io/date_algorithms.html#is_leap.
                          2. Optimized the CGregorianLeapFunctor::operator() method per the information at
                          https://howardhinnant.github.io/date_algorithms.html#is_leap.
                          3. Removed the weekday & wYearday variables from the DateS struct. The DateS struct now contains just the
                          Year, Month & Day triplet.
                          4. Renamed the GetDate method to "NumToDMY"
                          5. Reimplemented the CJulianGregorianCalendar::DaysInMonth method per the information at
                          https://howardhinnant.github.io/date_algorithms.html#last_day_of_month.
                          6. Reimplemented the CGregorianCalendar::DMYToNum method per the information at 
                          https://howardhinnant.github.io/date_algorithms.html#days_from_civil.
                          6. Reimplemented the CGregorianCalendar::NumToDMY method per the information at 
                          https://howardhinnant.github.io/date_algorithms.html#civil_from_days.
                          7. Fixed an integer overflow bug in the CMoslemCalendar*::NumToDMY methods. This bug was found while testing 
                          Moslem dates from the year -2000000 to 2000000 in the updated test app. In fact the Gregorian, Julian, 
                          Moslem & Hebrew calendars are now exercised from years -2000000 to 2000000 by the test app.
         PJN / 13-11-2018 1. Fixed a number of C++ core guidelines compiler warnings. These changes mean that the code will now only 
                          compile on VC 2017 or later on Windows.
                          2. Replaced enum with enum class throughout the codebase.
                          3. The code now allows construction of a CLTimeOfDay and CLLTimeOfDay which represents a time of day of 
                          23:59:60 which occurs when a UTC leap second occurs. Please note that if your OS exposes the concept of a 
                          leap second, then DTime+ will be able to accept this value, but roundtripping back to hours, minutes and 
                          seconds will give back midnight for the next day. Also note that timespan calculations will ignore leap 
                          seconds. If you would like to accurately calculate timespans taking into account leap seconds, then 
                          please see the author's AA+ framework at http://www.naughter.com/aa.html
         PJN / 08-06-2019 1. Updated copyright details
                          2. Updated the code to clean compile on VC 2019
         PJN / 13-10-2019 1. Fixed a number of compiler warnings when the code is compiled with VS 2019 Preview
         PJN / 16-05-2020 1. Updated copyright details
                          2. Fixed Clang-Tidy static code analysis warnings in the code.
         PJN / 22-05-2022 1. Updated copyright details.
                          2. Updated the code to use C++ uniform initialization for all variable declarations
         PJN / 03-06-2023 1. Updated copyright details.
                          2. Updated module to indicate that it needs to be compiled using /std:c++17. Thanks to Martin Richter
                          for reporting this issue.
         PJN / 30-10-2023 1. Made the operator- methods of CLDate const.
                          2. Made the operator- methods of CLLDate const.
                          3. Made CLDate::GetAsTM const.

Copyright (c) 1995 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#if _MSC_VER > 1000
#pragma once
#endif //#if _MSC_VER > 1000

#ifndef __DTIMEPLUS_H__
#define __DTIMEPLUS_H__

#ifdef _MSC_VER
#if _MSVC_LANG < 201703
#error DTime+ requires a minimum C++ language standard of /std:c++17
#endif //#if _MSVC_LANG < 201703
#endif //#ifdef _MSC_VER


//////////////////// Includes /////////////////////////////////////////////////

#include "DTime+Defines.h"
#include "DTime+JulianGregorianCalendar.h"
#include "DTime+JulianCalendar.h"
#include "DTime+GregorianCalendar.h"
#include "DTime+MoslemCalendar.h"
#include "DTime+HebrewCalendar.h"
#include "DTime+CDate.h"
#include "DTime+CLTimeOfDay.h"
#include "DTime+CLLTimeOfDay.h"
#include "DTime+CLTimeSpan.h"
#include "DTime+CLLTimeSpan.h"
#include "DTime+CLDate.h"
#include "DTime+CLLDate.h"
#include "DTime+JulianGregorianHolidays.h"
#include "DTime+MoslemHolidays.h"
#include "DTime+HebrewHolidays.h"

#endif //#ifndef __DTIMEPLUS_H__
