/*
Module : DTime+BaseCalendar.h
Purpose: Defines the interface for the DTime+ CBaseCalendar class
Created: PJN / 01-05-2006
History: PJN / 20-05-2006 1. Canonical DaysInMonth method now takes a Year and a Month parameter instead of a
                          month and a leap year boolean. This is required for support of the Hebrew calendar
                          where knowing whether a year is a leap year or not is not good enough to determine
                          the number of days in any month. This is because of the more complex nature of the 
                          Hebrew calendar and its' postponement rules for the Hebrew New Year (Rosh ha-Shannah).
                          Please note that this means you should examine all calls to this method in client
                          code to ensure it is updated correctly.
                          2. Canonical DaysInYear method layout has been updated to take a year instead of a
                          leap year boolean. The reason is as for item one above. Again you should examine all
                          calls to this method in client code to ensure it is updated correctly.
                          3. Canonical DaysSinceEndPreviousYear and DaysSinceFirstOfYear parameter layout 
                          has been updated. Both now take a year, month and day indicator. The reason is as 
                          for item one above. Again you should examine all calls to this method in client code 
                          to ensure it is updated correctly.

Copyright (c) 2006 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#if _MSC_VER > 1000
#pragma once
#endif //#if _MSC_VER > 1000

#ifndef __DTIMEPLUSBASECALENDAR_H__
#define __DTIMEPLUSBASECALENDAR_H__


//////////////////// Includes /////////////////////////////////////////////////

#include <cstdint>
#include "DTime+Defines.h"


//////////////////// Classes //////////////////////////////////////////////////

namespace DTimePlus
{

//This is the canonical class definition which defines the calendar traits class used by CDate, CLDate & CLLDate
class CBaseCalendar
{
public:
//Methods
  static bool DMYToNum(int32_t Year, Month month, uint16_t Day, int32_t& lDays, bool bDoAssert);
  static void NumToDMY(int32_t Days, int32_t& Year, uint16_t& Month, uint16_t& Day);
  static uint16_t DaysInMonth(int32_t Year, Month month);
  static uint16_t DaysInYear(int32_t Year);
  static bool IsLeap(int32_t Year);
  static uint16_t DaysSinceEndPreviousYear(int32_t Year, uint16_t month, uint16_t Day);
  static uint16_t DaysSinceFirstOfYear(int32_t Year, uint16_t month, uint16_t Day);
};

} //namespace DTimePlus

#endif //#ifndef __DTIMEPLUSBASECALENDAR_H__
