/*
Module : DTime+Defines.h
Purpose: Defines various enums and structs used by DTime+
Created: PJN / 01-05-2006

Copyright (c) 2006 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#if _MSC_VER > 1000
#pragma once
#endif //#if _MSC_VER > 1000

#ifndef __DTIMEPLUSDEFINES_H__
#define __DTIMEPLUSDEFINES_H__


//////////////////// Includes /////////////////////////////////////////////////

#include <string>
#include <cassert>

#undef __DTIMEPLUS_HAS_SYS_TIME
#ifndef _WIN32
  #ifdef __has_include
    #if __has_include(<sys/time.h>)
      #define __DTIMEPLUS_HAS_SYS_TIME
      #include <sys/time.h>
    #endif //#if __has_include(<sys/time.h>)
  #endif //#ifdef __has_include
#endif //#ifndef _WIN32


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define DTIME_DEBUG_ONLY(f) (f)
#else
#define DTIME_DEBUG_ONLY(f) ((void)0)
#endif //#ifdef _DEBUG

#define DTIME_ASSERT(f) DTIME_DEBUG_ONLY( assert(f) )

#ifdef _MSC_VER
#if defined(_PREFAST_) || defined (_DEBUG)
#define DTIME_ASSUME(cond) do { const bool __dtime_condVal=!!(cond); DTIME_ASSERT(__dtime_condVal); __analysis_assume(__dtime_condVal); } while(0)
#else
#define DTIME_ASSUME(cond) ((void)0)
#endif //#if defined(_PREFAST_) || defined (_DEBUG)
#else
#define DTIME_ASSUME(cond) ((void)0)
#endif //#ifdef _MSC_VER

#ifndef _WIN32
#define TCHAR char
#ifdef _UNICODE
#define _T(x) L ## x
#define _ttoi _wtoi
#define _ttol _wtol
#define _tprintf wprintf
#define _tcstok _wcstok
#else
#define _T(x) x
#define _ttoi atoi
#define _ttol atol
#define _tprintf printf
#define _tcstok strtok
#endif //#ifdef _UNICODE
#define UNREFERENCED_PARAMETER(P) ((void)P)
#endif //#ifndef _WIN32

#ifndef _HAS_DTIME_NODISCARD
#ifndef __has_cpp_attribute
#define _HAS_DTIME_NODISCARD 0
#elif __has_cpp_attribute(nodiscard) >= 201603L
#define _HAS_DTIME_NODISCARD 1
#else
#define _HAS_DTIME_NODISCARD 0
#endif
#endif //#ifndef _HAS_DTIME_NODISCARD

#if _HAS_DTIME_NODISCARD
#define DTIME_NODISCARD [[nodiscard]]
#else
#define DTIME_NODISCARD
#endif //#if _HAS_DTIME_NODISCARD


//////////////////// Enums / Structs //////////////////////////////////////////

namespace DTimePlus
{

enum class Month
{
  January = 1,
  February = 2,
  March = 3,
  April = 4,
  May = 5,
  June = 6,
  July = 7,
  August = 8,
  September = 9,
  October = 10,
  November = 11,
  December = 12
};

enum class DayOfWeek
{
  Sunday = 1,
  Monday = 2,
  Tuesday = 3,
  Wednesday = 4,
  Thursday = 5,
  Friday = 6,
  Saturday = 7,
  FirstDayOfWeek = Sunday,
  LastDayOfWeek  = Saturday
};

enum class DateEpoch
{
  EPOCH_JD = 0,
  EPOCH_MJD = 1,
  EPOCH_1900 = 2,
  EPOCH_1950 = 3,
  EPOCH_CTIME = 4,
  EPOCH_2000 = 5,
  EPOCH_GREG = 6
};

struct DateS
{
  int32_t Year;
  uint16_t Month;
  uint16_t Day;
};

struct DateLS
{
  int32_t Year;
  uint16_t Month;
  uint16_t Day;
  uint16_t Hour;
  uint16_t Minute;
  uint16_t Second;
};

struct DateLLS
{
  int32_t Year;
  uint16_t Month;
  uint16_t Day;
  uint16_t Hour;
  uint16_t Minute;
  double Second;
};

#ifdef _UNICODE
using String = std::wstring;
using dt_ostringstream = std::wostringstream;
#else
using String = std::string;
using dt_ostringstream = std::ostringstream;
#endif //#ifdef _UNICODE

} //namespace DTimePlus

#endif //#ifndef __DTIMEPLUSDEFINES_H__
