/*
Module : DTime+HebrewHolidays.H
Purpose: Defines the interface for the DTime+ CHebrewHolidays class which defines various methods
         to create holidays in the Hebrew calendar
Created: PJN / 29-05-2006

Copyright (c) 2006 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#if _MSC_VER > 1000
#pragma once
#endif //#if _MSC_VER > 1000

#ifndef __DTIMEPLUSHEBREWHOLIDAYS_H__
#define __DTIMEPLUSHEBREWHOLIDAYS_H__


//////////////////// Includes /////////////////////////////////////////////////

#include <vector>
#include "DTime+HebrewCalendar.h"
#include "DTime+GregorianCalendar.h"
#include "DTime+CDate.h"


//////////////////// Classes //////////////////////////////////////////////////

namespace DTimePlus
{

class CRoshHaShanah1Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Tishrei), 1};
  }
};

class CRoshHaShanah2Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Tishrei), 2};
  }
};

class CYomKippurFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Tishrei), 10};
  }
};

class CSukkotFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Tishrei), 15};
  }
};

class CHoshanahRabahFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Tishrei), 21};
  }
};

class CSheminiAzeretFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Tishrei), 22};
  }
};

class CCSimhatTorahOutsideIsraelFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Tishrei), 23};
  }
};

class CTaanitBechorimFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> taanitBechorim{Year, static_cast<uint16_t>(HebrewMonth::Nissan), 14};
    if (taanitBechorim.GetDayOfWeek() == DayOfWeek::Saturday)
      return taanitBechorim - 1;
    else
      return taanitBechorim;
  }
};

class CErevPesachFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Nissan), 14};
  }
};

class CPesachFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Nissan), 15};
  }
};

class CShviiPesachFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Nissan), 21};
  }
};

class CShavuotFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Sivan), 6};
  }
};

class CHanukkah1Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Kislev), 25};
  }
};

class CHanukkah2Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CHanukkah1Functor()(Year) + 1;
  }
};

class CHanukkah3Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CHanukkah1Functor()(Year) + 2;
  }
};

class CHanukkah4Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CHanukkah1Functor()(Year) + 3;
  }
};

class CHanukkah5Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CHanukkah1Functor()(Year) + 4;
  }
};

class CHanukkah6Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CHanukkah1Functor()(Year) + 5;
  }
};

class CHanukkah7Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CHanukkah1Functor()(Year) + 6;
  }
};

class CHanukkah8Functor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CHanukkah1Functor()(Year) + 7;
  }
};

class CTuBShevatFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Shevat), 15};
  }
};

class CPurimFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, CHebrewCalendar::LastMonthOfYear(CHebrewCalendar::IsLeap(Year)), 14};
  }
};

class CTzomGedaliahFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> tzomGedaliah{Year, static_cast<uint16_t>(HebrewMonth::Tishrei), 3};
    if (tzomGedaliah.GetDayOfWeek() == DayOfWeek::Saturday)
      return tzomGedaliah + 1;
    else
      return tzomGedaliah;
  }
};

class CTzomTevetFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> tzomTevet{Year, static_cast<uint16_t>(HebrewMonth::Tevet), 10};
    if (tzomTevet.GetDayOfWeek() == DayOfWeek::Saturday)
      return tzomTevet + 1;
    else
      return tzomTevet;
  }
};

class CTaanitEstherFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> purim{CPurimFunctor()(Year)};
    if (purim.GetDayOfWeek() == DayOfWeek::Sunday)
      return purim - 3;
    else
      return purim - 1;
  }
};

class CTzomTammuzFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> tzomTammuz{Year, static_cast<uint16_t>(HebrewMonth::Tammuz), 17};
    if (tzomTammuz.GetDayOfWeek() == DayOfWeek::Saturday)
      return tzomTammuz + 1;
    else
      return tzomTammuz;
  }
};

class CTishahBeAvFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> tishahBeAv{Year, static_cast<uint16_t>(HebrewMonth::Av), 9};
    if (tishahBeAv.GetDayOfWeek() == DayOfWeek::Saturday)
      return tishahBeAv + 1;
    else
      return tishahBeAv;
  }
};

class CShushanPurimFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> shushanPurim{CPurimFunctor()(Year)};
    if (shushanPurim.GetDayOfWeek() == DayOfWeek::Saturday)
      return shushanPurim + 1;
    else
      return shushanPurim;
  }
};

class CYomHaShoahFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> yomHaShoah{Year, static_cast<uint16_t>(HebrewMonth::Nissan), 27};
    if (yomHaShoah.GetDayOfWeek() == DayOfWeek::Sunday)
      return yomHaShoah + 1;
    else
      return yomHaShoah;
  }
};

class CYomHaZikkaronFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    const CDate<CHebrewCalendar> yomHaZikkaron{Year, static_cast<uint16_t>(HebrewMonth::Iyyar), 4};
    const DayOfWeek dow{yomHaZikkaron.GetDayOfWeek()};
    if (dow == DayOfWeek::Thursday)
      return yomHaZikkaron - 1; //Move to the prior Wednesday
    else if (dow == DayOfWeek::Friday)
      return yomHaZikkaron - 2; //Move to the prior Wednesday
    else
      return yomHaZikkaron;
  }
};

class CYomHaatzmautFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CYomHaZikkaronFunctor()(Year) + 1;
  }
};

class CPesachShenniFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Iyyar), 14};
  }
};

class CYomYerushalaiymFunctor
{
public:
  CDate<CHebrewCalendar> operator()(int32_t Year) noexcept
  {
    return CDate<CHebrewCalendar>{Year, static_cast<uint16_t>(HebrewMonth::Iyyar), 28};
  }
};

//Function which returns all occurrences of a specific date in the Hebrew calendar (generated by a functor)
//in a specified Gregorian year. Note that because we are using the flexibility of functors, this function 
//is reusable where we want to calculate Hebrew holidays which do not occur on simple fixed days in the 
//Hebrew calendar
template<class TDATEFUNCTOR>
static void HewbrewOccurrencesInGregorianYear(int32_t Year, std::vector<CDate<CGregorianCalendar> >& Occurrences)
{
  //Empty out the array before we add any values to it
  Occurrences.clear();

  const CDate<CGregorianCalendar> jan1{Year, static_cast<uint16_t>(Month::January), 1};
  const int32_t jan1value{jan1.GetValue()};
  const CDate<CGregorianCalendar> dec31{Year, static_cast<uint16_t>(Month::December), 31};
  const int32_t dec31value{dec31.GetValue()};
  const int32_t y{CDate<CHebrewCalendar>(jan1value, DateEpoch::EPOCH_GREG).GetYear()};

  TDATEFUNCTOR tdate;
  CDate<CHebrewCalendar> date = tdate(y - 2);
  int32_t datevalue{date.GetValue()};
  if ((jan1value <= datevalue) && (datevalue <= dec31value))
    Occurrences.emplace_back(datevalue, DateEpoch::EPOCH_GREG);

  date = tdate(y - 1);
  datevalue = date.GetValue();
  if ((jan1value <= datevalue) && (datevalue <= dec31value))
  {
#ifdef _MSC_VER
#pragma warning(suppress: 26489)
#endif //#ifdef _MSC_VER
    Occurrences.emplace_back(datevalue, DateEpoch::EPOCH_GREG);
  }

  date = tdate(y);
  datevalue = date.GetValue();
  if ((jan1value <= datevalue) && (datevalue <= dec31value))
  {
#ifdef _MSC_VER
#pragma warning(suppress: 26489)
#endif //#ifdef _MSC_VER
    Occurrences.emplace_back(datevalue, DateEpoch::EPOCH_GREG);
  }

  date = tdate(y + 1);
  datevalue = date.GetValue();
  if ((jan1value <= datevalue) && (datevalue <= dec31value))
  {
#ifdef _MSC_VER
#pragma warning(suppress: 26489)
#endif //#ifdef _MSC_VER
    Occurrences.emplace_back(datevalue, DateEpoch::EPOCH_GREG);
  }

  date = tdate(y + 2);
  datevalue = date.GetValue();
  if ((jan1value <= datevalue) && (datevalue <= dec31value))
  {
#ifdef _MSC_VER
#pragma warning(suppress: 26489)
#endif //#ifdef _MSC_VER
    Occurrences.emplace_back(datevalue, DateEpoch::EPOCH_GREG);
  }
};

class CHebrewHolidays
{
public:
  inline static bool Omer(const CDate<CHebrewCalendar> date, int32_t& OmerDayNumber) noexcept
  {
    int32_t c{date.GetValue() - CPesachFunctor()(date.GetYear()).GetValue()};
    if ((1 <= c) && (c <= 49))
    {
      //Calculate the output parameter
      OmerDayNumber = c;
      return true;
    }
    else
      return false; //specified date is not in Omer
  }
};

} //namespace DTimePlus

#endif //#ifndef __DTIMEPLUSHEBREWHOLIDAYS_H__
