#ifdef _MSC_VER
#include "stdafx.h"
#endif //#ifdef _MSC_VER

#include "DTimeTest.h"
#include "DTime+.h"
#include <iostream>
#include <cstdlib>

#ifdef _AFX
#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG

#pragma warning(suppress: 26426)
CWinApp theApp;
#endif //#ifdef _AFX

using namespace DTimePlus;


int main()
{
#ifdef _AFX
  //initialize MFC and print and error on failure
  if (!AfxWinInit(GetModuleHandle(nullptr), nullptr, GetCommandLine(), 0))
  {
    _tprintf(_T("Fatal Error: MFC initialization failed\n"));
    return 1;
  }
#endif //#ifdef _AFX

  const CLDate<CGregorianCalendar> startDate{CLDate<CGregorianCalendar>::CurrentSystemTime()};

  //Seed the random number generator
#ifdef _MSC_VER
#pragma warning(suppress: 26472)
#endif //#ifdef _MSC_VER
  srand(static_cast<unsigned int>(time(nullptr)));

  //Display some intro text
  _tprintf(_T("This program is best run under a debugger so that\n"));
  _tprintf(_T("the code can be single stepped and stepped into etc.\n"));

  DTIME_ASSERT(DTimePlus::lmod(9, 5) == 4);
  DTIME_ASSERT(DTimePlus::lmod(-9, 5) == 1);
  DTIME_ASSERT(DTimePlus::lmod(9, -5) == -1);
  DTIME_ASSERT(DTimePlus::lmod(-9, -5) == -4);

  //Test out support for handling dates with leap seconds in them
  const CLDate<CGregorianCalendar> leap1{2018, 7, 31, 23, 59, 60};
  DateLS ls{leap1.GetDate()};
  UNREFERENCED_PARAMETER(ls);
  const CLLDate<CGregorianCalendar> leap2{2018, 7, 31, 23, 59, 60};
  DateLLS lls{leap2.GetDate()};
  UNREFERENCED_PARAMETER(lls);
#ifdef _WIN32
  SYSTEMTIME st{};
  st.wYear = 2018;
  st.wMonth = 7;
  st.wDay = 31;
  st.wHour = 23;
  st.wMinute = 59;
  st.wSecond = 60;
  const CLDate<CGregorianCalendar> leap3{st};
  ls = leap3.GetDate();
  const CLLDate<CGregorianCalendar> leap4{st};
  lls = leap4.GetDate();
#endif //#ifdef _WIN32
  const CLTimeOfDay<> leap5{86400};
  uint16_t h{leap5.GetHours()};
  UNREFERENCED_PARAMETER(h);
  uint16_t m{leap5.GetMinutes()};
  UNREFERENCED_PARAMETER(m);
  const uint16_t s1{leap5.GetSeconds()};
  UNREFERENCED_PARAMETER(s1);
  const CLLTimeOfDay<> leap6{86400.0};
  h = leap6.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  m = leap6.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  double ss2{leap6.GetSeconds()};
  UNREFERENCED_PARAMETER(ss2);

  //Test of the CDate class
  const CDate<CGregorianCalendar> xstr{1996, static_cast<uint16_t>(Month::January), 1};
  String sString{xstr.Format(_T("%Y-%m-%d"))};

  //Test of the calendar constructor parameters
  CDate<CGregorianCalendar> x{1582, static_cast<uint16_t>(Month::October), 15};
  CDate<CJulianCalendar> p{x.AsJulianCDate()};
  sString = p.Format(_T("%Y-%m-%d"));
  CDate<CGregorianCalendar> p2{x.AsGregorianCDate()};
  sString = p2.Format(_T("%Y-%m-%d"));
  CDate<CJulianCalendar> x2{CDate<CJulianCalendar>(1582, static_cast<uint16_t>(Month::October), 15)};
  p = x2.AsJulianCDate();
  sString = p.Format(_T("%Y-%m-%d"));
  p2 = x.AsGregorianCDate();
  sString = p2.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 15};
  p = x.AsJulianCDate();
  sString = p.Format(_T("%Y-%m-%d"));
  p2 = x.AsGregorianCDate();
  sString = p2.Format(_T("%Y-%m-%d"));
  x2 = CDate<CJulianCalendar>{1582, static_cast<uint16_t>(Month::October), 4};
  p = x2.AsJulianCDate();
  sString = p.Format(_T("%Y-%m-%d"));
  p2 = x.AsGregorianCDate();
  sString = p2.Format(_T("%Y-%m-%d"));
  x2 = CDate<CJulianCalendar>{1582, static_cast<uint16_t>(Month::October), 4};
  p = x2.AsJulianCDate();
  sString = p.Format(_T("%Y-%m-%d"));
  p2 = x.AsGregorianCDate();
  sString = p2.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 4};
  p = x.AsJulianCDate();
  sString = p.Format(_T("%Y-%m-%d"));
  p2 = x.AsGregorianCDate();
  sString = p2.Format(_T("%Y-%m-%d"));
#ifdef _WIN32
  sString = p2.Format(CDate<>::GetWindowsFormatString());
#endif //#ifdef _WIN32

  //first all the constructors
  x = CDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 15}; //GDN = 0
  x = CDate<CGregorianCalendar>{1992, static_cast<uint16_t>(Month::October), 10}; //GDN = 149745
  x2 = CDate<CJulianCalendar>{-4712, static_cast<uint16_t>(Month::January), 1}; //GDN = -2299161
  x2 = CDate<CJulianCalendar>{1582, static_cast<uint16_t>(Month::October), 4}; //GDN = -1;

  x = CDate<CGregorianCalendar>{2006, static_cast<uint16_t>(Month::April), 22}; //GDN = 154687;
  DayOfWeek dow{x.GetDayOfWeek()}; //NOLINT(clang-analyzer-deadcode.DeadStores)
  sString = x.Format(_T("%Y-%m-%d"));

  String s;
#ifdef _WIN32
  //Construction using the Win32 SYSTEMTIME structure
  st = {};
  st.wYear = 1582;
  st.wMonth = 10;
  st.wDay = 15;
  x = CDate<CGregorianCalendar>{st};
  s = x.Format(_T("%Y-%m-%d"));
#endif //#ifdef _WIN32

#ifdef _WIN32
  //Construction using the Win32 FILETIME structure
  GetLocalTime(&st);
  FILETIME ft;
  SystemTimeToFileTime(&st, &ft);
  x = CDate<CGregorianCalendar>{ft};
  s = x.Format(_T("%Y-%m-%d"));
#endif //#ifdef _WIN32

#ifdef _AFX
  //Construction using a CTime
  const CTime t{CTime::GetCurrentTime()};
  x = CDate<CGregorianCalendar>{t};
  s = x.Format(_T("%Y-%m-%d"));
#endif //#ifdef _AFX

  //Construction using the direct count of days
  x = CDate<CGregorianCalendar>{0, DateEpoch::EPOCH_JD}; //GDN = -2299161
  x = CDate<CGregorianCalendar>{0, DateEpoch::EPOCH_GREG}; //GDN = 0
  x = CDate<CGregorianCalendar>{0, DateEpoch::EPOCH_MJD}; //GDN = 100839
  x = CDate<CGregorianCalendar>{0, DateEpoch::EPOCH_1900}; //GDN = 115860
  x = CDate<CGregorianCalendar>{0, DateEpoch::EPOCH_1950}; //GDN = 134122
  x = CDate<CGregorianCalendar>{0, DateEpoch::EPOCH_CTIME}; //GDN = 141427
  x = CDate<CGregorianCalendar>{0, DateEpoch::EPOCH_2000}; //GDN = 152384

#ifdef _AFX
  //CTime Construction
  const CTime z{1970, 1, 2, 0, 0, 0};
  x = CDate<CGregorianCalendar>{z}; //GDN = 141428
  s = x.Format(_T("%Y-%m-%d"));
#endif //#ifdef _AFX

#ifdef _AFX
  //COleDateTime Construction
  COleDateTime dt{1992, static_cast<uint16_t>(Month::October), 10, 0, 0, 0};
  x = CDate<CGregorianCalendar>{dt}; //GDN = 149745
  s = x.Format(_T("%Y-%m-%d"));
  dt = COleDateTime{1970, 1, 2, 0, 0, 0};
  x = CDate<CGregorianCalendar>{dt}; //GDN = 141428
  s = x.Format(_T("%Y-%m-%d"));
#endif //#ifdef _AFX

  //String constructors
  x = CDate<CGregorianCalendar>{_T("20/03/96"), _T("dmy2/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("20/03/46"), _T("dmy2/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("2/03/96"), _T("mdy2/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("2-03-96"), _T("mdy2-")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("20/03/1496"), _T("dmy4/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("20/03/2146"), _T("dmy4/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("2/03/1896"), _T("mdy4/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("2-03-1496"), _T("mdy4-")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("1496/20/03"), _T("ydm4/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("2146/20/03"), _T("ydm4/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("1896/2/03"), _T("ymd4/")};
  s = x.Format(_T("%Y-%m-%d"));
  x = CDate<CGregorianCalendar>{_T("1496-2-03"), _T("ymd4-")};
  s = x.Format(_T("%Y-%m-%d"));

  //Static Constructors
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  s = x.Format(_T("%Y-%m-%d"));
  const int32_t nCurrentYear{x.GetDate().Year};
  UNREFERENCED_PARAMETER(nCurrentYear);
  _tprintf(_T("The current local date is %s\n"), s.c_str());
  s = x.AsJulianCDate().Format(_T("%Y-%m-%d"));
  _tprintf(_T("The current local date (in the Julian Calendar) is %s\n"), s.c_str());
  p = CDate<CJulianCalendar>::CurrentLocalDate();
  s = p.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The current local date (in the Julian Calendar) is %s\n"), s.c_str());

  x = CDate<CGregorianCalendar>::CurrentSystemDate();
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The current system date is %s\n"), s.c_str());
  s = x.AsJulianCDate().Format(_T("%Y-%m-%d"));
  _tprintf(_T("The current system date (in the Julian Calendar) is %s\n"), s.c_str());
  p = CDate<CJulianCalendar>::CurrentSystemDate();
  s = p.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The current system date (in the Julian Calendar) is %s\n"), s.c_str());

  x2 = CDate<CJulianCalendar>::JDEpoch();
  s = x2.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The Julian Day Epoch is %s\n"), s.c_str());
  x = CDate<CGregorianCalendar>::MJDEpoch();
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The Modified Julian Day Epoch is %s\n"), s.c_str());
  x = CDate<CGregorianCalendar>::Epoch1900();
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The 1900 Epoch is %s\n"), s.c_str());
  x = CDate<CGregorianCalendar>::Epoch1950();
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The 1950 Epoch is %s\n"), s.c_str());
  x = CDate<CGregorianCalendar>::EpochCTime();
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The CTime Epoch is %s\n"), s.c_str());
  x = CDate<CGregorianCalendar>::Epoch2000();
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The 2000 Epoch is %s\n"), s.c_str());
  x = CDate<CGregorianCalendar>::GregorianEpoch();
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The Gregorian Epoch is %s\n"), s.c_str());

  //Try the static operations
  DTIME_ASSERT(CGregorianCalendar::IsLeap(2000));
  DTIME_ASSERT(!CGregorianCalendar::IsLeap(1900));
  DTIME_ASSERT(!CGregorianCalendar::IsLeap(1902));
  DTIME_ASSERT(CJulianCalendar::IsLeap(1400));
  DTIME_ASSERT(CJulianCalendar::IsLeap(1300));
  DTIME_ASSERT(!CJulianCalendar::IsLeap(1302));
  DTIME_ASSERT(CGregorianCalendar::DaysInYear(2000) == 366);
  DTIME_ASSERT(CGregorianCalendar::DaysInYear(1998) == 365);
  DTIME_ASSERT(CGregorianCalendar::DaysInMonth(2000, static_cast<uint16_t>(Month::January)) == 31);
  DTIME_ASSERT(CGregorianCalendar::DaysInMonth(2000, static_cast<uint16_t>(Month::February)) == 29);
  DTIME_ASSERT(CGregorianCalendar::DaysInMonth(2001, static_cast<uint16_t>(Month::February)) == 28);
  DTIME_ASSERT(CGregorianCalendar::DaysInMonth(2000, static_cast<uint16_t>(Month::June)) == 30);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2000, static_cast<uint16_t>(Month::January), 1) == 1);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2001, static_cast<uint16_t>(Month::January), 1) == 1);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2000, static_cast<uint16_t>(Month::December), 31) == 366);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2001, static_cast<uint16_t>(Month::December), 31) == 365);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2000, static_cast<uint16_t>(Month::July), 13) == 195);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2001, static_cast<uint16_t>(Month::July), 13) == 194);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2000, static_cast<uint16_t>(Month::December), 31) == 365);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2001, static_cast<uint16_t>(Month::December), 31) == 364);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2000, static_cast<uint16_t>(Month::July), 13) == 194);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2001, static_cast<uint16_t>(Month::July), 13) == 193);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2000, static_cast<uint16_t>(Month::January), 1) == 0);
  DTIME_ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2001, static_cast<uint16_t>(Month::January), 1) == 0);

  //Construction using the WeekOfMonth parameter
  x = CDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::September), 3, DayOfWeek::Friday};
  DTIME_ASSERT(x == CDate<CGregorianCalendar>(1995, static_cast<uint16_t>(Month::September), 15));
  x = CDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::September), 1, DayOfWeek::Saturday};
  DTIME_ASSERT(x == CDate<CGregorianCalendar>(1995, static_cast<uint16_t>(Month::September), 2));
  x = CDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::September), 1, DayOfWeek::Tuesday};
  DTIME_ASSERT(x == CDate<CGregorianCalendar>(1995, static_cast<uint16_t>(Month::September), 5));
  x = CDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::September), 5, DayOfWeek::Sunday};
  DTIME_ASSERT(x == CDate<CGregorianCalendar>(1995, static_cast<uint16_t>(Month::September), 24));
  x = CDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::September), 5, DayOfWeek::Friday};
  DTIME_ASSERT(x == CDate<CGregorianCalendar>(1995, static_cast<uint16_t>(Month::September), 29));

  //Test out the holidays
  CDate<> hol1{CJulianGregorianHolidays<>::NewYearsDay(nCurrentYear)};
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("New Years Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::StValentinesDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Saint Valentines Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::AshWednesday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Ash Wednesday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::StPatricksDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Saint Patricks Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::GoodFriday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Good Friday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::EasterSunday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Easter Sunday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::EasterMonday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Easter Monday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::CanadaDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Canada Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::BastilleDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Bastille Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::ChristmasDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Christmas Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::MartinLutherKingDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Martin Luther King Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::AbrahamLincolnsBirthday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Abraham Lincolns birthday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::GeorgeWashingtonsBirthday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("George Washingtons birthday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USPresidentsDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Presidents Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USIndependenceDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Independence Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USArmedForcesDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Armed Forces Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USMemorialDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Memorial Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USFlagDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Flag Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USLaborDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US & Canada Labor Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USColumbusDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Columbus Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::CanadaThanksgivingDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Canada Thanksgiving Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USElectionDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Election Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USVeteransDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Veterans Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::USThanksgivingDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("US Thanksgiving Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::PalmSunday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Palm Sunday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::MaundyThursday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Maundy Thursday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::FeastOfTheAscension(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Feast Of the Ascension in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::FeastOfTheAnnunciation(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Feast of the Annunciation in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::Epiphany(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Epiphany in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::Candlemas(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Candlemas in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::ShroveTuesday(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Shrove Tuesday in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::Pentecost(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Pentecost in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::UKBoxingDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("UK & Canada Boxing Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::StAndrewsDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Saint Andrews Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::StPaulsDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Saint Pauls Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::StGeorgesDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Saint Georges Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::StDavidsDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Saint Davids Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::StStephensDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Saint Stephens Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::GroundhogDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Groundhog Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::FeastOfTheAssumption(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Feast of the Assumption in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::AllSaintsDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("All Saints Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::AllSoulsDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("All Souls Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::FeastOfTheImmaculateConception(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Feast of the Immaculate Conception in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::AprilFoolsDay(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("April Fools Day in %d is %s\n"), nCurrentYear, s.c_str());
  hol1 = CJulianGregorianHolidays<>::Halloween(nCurrentYear);
  s = hol1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Halloween in %d is %s\n"), nCurrentYear, s.c_str());

  CDate<CMoslemCalendarCivil16> mdate4{CMoslemNewYearFunctor<CMoslemCalendarCivil16>()(1427)};
  s = mdate4.Format(_T("%Y-%m-%d"));
  mdate4 = CAshuraFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format(_T("%Y-%m-%d"));
  mdate4 = CNawlidAnNabiFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format(_T("%Y-%m-%d"));
  mdate4 = CLailatAlMiRajFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format(_T("%Y-%m-%d"));
  mdate4 = CLailatAlBaraaFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format(_T("%Y-%m-%d"));
  mdate4 = CRamdadanFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format(_T("%Y-%m-%d"));
  mdate4 = CLailatAlKadrFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format(_T("%Y-%m-%d"));
  mdate4 = CIdAlFitrFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format(_T("%Y-%m-%d"));
  mdate4 = CIdAlAdhaFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format(_T("%Y-%m-%d"));

  std::vector<CDate<CGregorianCalendar> > occurrences;
  MoslemOccurrencesInGregorianYear<CIdAlAdhaFunctor<CMoslemCalendarCivil16>, CMoslemCalendarCivil16>(nCurrentYear, occurrences);
  _tprintf(_T("IdAlAdha in the Moslem Civil16 calendar occurrences in the Gregorian year %d\n"), nCurrentYear);
  for (const auto& occurrence : occurrences)
    _tprintf(_T("%s\n"), occurrence.Format(_T("%Y-%m-%d")).c_str());
  const uint16_t nDaysInYear{mdate4.DaysInYear()};
  UNREFERENCED_PARAMETER(nDaysInYear);

  CDate<CHebrewCalendar> hdate1{CRoshHaShanah1Functor()(5766)};
  s = hdate1.Format(_T("%Y-%m-%d"));
  s = hdate1.AsGregorianCDate().Format(_T("%Y-%m-%d"));
  hdate1 = CYomKippurFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CSukkotFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHoshanahRabahFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CSheminiAzeretFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CCSimhatTorahOutsideIsraelFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CPesachFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CShviiPesachFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHanukkah1Functor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHanukkah2Functor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHanukkah3Functor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHanukkah4Functor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHanukkah5Functor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHanukkah6Functor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHanukkah7Functor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CHanukkah8Functor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CPurimFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  s = hdate1.AsGregorianCDate().Format(_T("%Y-%m-%d"));

  hdate1 = CTzomGedaliahFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CTzomTevetFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CTaanitEstherFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CTzomTammuzFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CTishahBeAvFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  hdate1 = CShushanPurimFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  int32_t Year{0};
  for (Year=5760; Year<=5800; Year++)
  {
    hdate1 = CYomHaShoahFunctor()(Year);
    s = hdate1.Format(_T("%Y-%m-%d"));
  }
  for (Year=5760; Year<5800; Year++)
  {
    hdate1 = CYomHaZikkaronFunctor()(Year);
    s = hdate1.Format(_T("%Y-%m-%d"));
  }

  hdate1 = CShavuotFunctor()(5766);
  s = hdate1.Format(_T("%Y-%m-%d"));
  int32_t OmerDayNumber{0};
  bool bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber);
  UNREFERENCED_PARAMETER(bOmer);
  hdate1 -= 3;
  s = hdate1.Format(_T("%Y-%m-%d"));
  bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hdate1 -= 45;
  s = hdate1.Format(_T("%Y-%m-%d"));
  bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hdate1 -= 1;
  s = hdate1.Format(_T("%Y-%m-%d"));
  bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hdate1 -= 1;
  s = hdate1.Format(_T("%Y-%m-%d"));
  bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber); //NOLINT(clang-analyzer-deadcode.DeadStores)

  HewbrewOccurrencesInGregorianYear<CPesachFunctor>(nCurrentYear, occurrences);
  _tprintf(_T("Pesach (Passover) in the Hebrew calendar occurrences in the Gregorian year %d\n"), nCurrentYear);
  for (const auto& occurrence : occurrences)
    _tprintf(_T("%s\n"), occurrence.Format(_T("%Y-%m-%d")).c_str());

  HewbrewOccurrencesInGregorianYear<CTzomTevetFunctor>(1982, occurrences);
  _tprintf(_T("Tzom Tevet in the Hebrew calendar occurrences in the Gregorian year %d\n"), 1982);
  for (const auto& occurrence : occurrences)
    _tprintf(_T("%s\n"), occurrence.Format(_T("%Y-%m-%d")).c_str());
  HewbrewOccurrencesInGregorianYear<CTzomTevetFunctor>(1984, occurrences);
  _tprintf(_T("Tzom Tevet in the Hebrew calendar occurrences in the Gregorian year %d\n"), 1984);
  for (const auto& occurrence : occurrences)
    _tprintf(_T("%s\n"), occurrence.Format(_T("%Y-%m-%d")).c_str());
  HewbrewOccurrencesInGregorianYear<CTzomTevetFunctor>(nCurrentYear, occurrences);
  _tprintf(_T("Tzom Tevet in the Hebrew calendar occurrences in the Gregorian year %d\n"), nCurrentYear);
  for (const auto& occurrence : occurrences)
    _tprintf(_T("%s\n"), occurrence.Format(_T("%Y-%m-%d")).c_str());

  int32_t lYear{CDate<CGregorianCalendar>::CurrentLocalDate().GetYear()};
  uint16_t wMonth{0};
  uint16_t wDay{0};
  CGregorianCalendar::EasterSunday(lYear, wMonth, wDay);
  x = CDate<CGregorianCalendar>{lYear, wMonth, wDay};
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Easter Sunday occurs on %s this Gregorian year\n"), s.c_str());

  CJulianCalendar::EasterSunday(lYear, wMonth, wDay);
  p = CDate<CJulianCalendar>{lYear, wMonth, wDay};
  s = p.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Easter Sunday occurs on %s this Julian year\n"), s.c_str());


  //Test out the Hebrew calendar support
  const CDate<CJulianCalendar> hepoch{-3760, static_cast<uint16_t>(Month::October), 7};

  hdate1 = CDate<CHebrewCalendar>{4682, static_cast<uint16_t>(HebrewMonth::Nissan), 15};
  s = hdate1.AsJulianCDate().Format(_T("%Y-%m-%d"));
  const DateS ds2{hdate1.GetDate()};
  UNREFERENCED_PARAMETER(ds2);
  const double Molah{CHebrewCalendar::Molad(4682, static_cast<uint16_t>(HebrewMonth::Tishrei))};
  const CLDate<CJulianCalendar> moladD{Molah, DateEpoch::EPOCH_GREG};
  s = moladD.Format(_T("%Y-%m-%d %H:%M:%S"));
  const double Molah2{CHebrewCalendar::Molad(4683, static_cast<uint16_t>(HebrewMonth::Tishrei))};
  const CLDate<CJulianCalendar> moladD2{Molah2, DateEpoch::EPOCH_GREG};
  s = moladD2.Format(_T("%Y-%m-%d %H:%M:%S"));

  const int32_t HebrewNewYear{CHebrewCalendar::HebrewNewYear(4682)};
  UNREFERENCED_PARAMETER(HebrewNewYear);
  const uint16_t wDelay{CHebrewCalendar::HebrewNewYearDelay(4682)};
  UNREFERENCED_PARAMETER(wDelay);
  _tprintf(_T("There are %d days in the Hebrew year 4682 A.M.\n"), static_cast<int>(CHebrewCalendar::DaysInYear(4682)));
  if (CHebrewCalendar::IsLeap(4682))
    _tprintf(_T("Hebrew year 4682 A.M is a leap year.\n"));
  else
    _tprintf(_T("Hebrew year 4682 A.M is a not leap year.\n"));
  const int32_t nHebrewElapsedDays{CHebrewCalendar::HebrewCalendarElapsedDays(4682)};
  UNREFERENCED_PARAMETER(nHebrewElapsedDays);
  const bool bLongCheshvan{CHebrewCalendar::LongCheshvan(4682)};
  if (bLongCheshvan)
    _tprintf(_T("Cheshvan is 30 days in length in the Hebrew year 4682 A.M\n"));
  else
    _tprintf(_T("Cheshvan is 29 days in length in the Hebrew year 4682 A.M\n"));
  const bool bShortCheshvan{CHebrewCalendar::ShortCheshvan(4682)};
  UNREFERENCED_PARAMETER(bShortCheshvan);
  const bool bLongKislev{CHebrewCalendar::LongKislev(4682)};
  if (bLongKislev)
    _tprintf(_T("Kislev is 30 days in length in the Hebrew year 4682 A.M\n"));
  else
    _tprintf(_T("Kislev is 30 days in length in the Hebrew year 4682 A.M\n"));
  const bool bShortKislev{CHebrewCalendar::ShortKislev(4682)};
  UNREFERENCED_PARAMETER(bShortKislev);
  _tprintf(_T("Hebrew Months for the year 4682 A.M.\n"));
  for (auto i=static_cast<uint16_t>(HebrewMonth::Nissan); i<=CHebrewCalendar::LastMonthOfYear(CHebrewCalendar::IsLeap(4682)); i++)
    _tprintf(_T("Month: %s %s %d\n"), CHebrewCalendar::GetFullStringMonth(i).c_str(), CHebrewCalendar::GetAbrStringMonth(i).c_str(), static_cast<int>(CHebrewCalendar::DaysInMonth(4682, i)));
  _tprintf(_T("Hebrew days of the week\n"));
  for (auto i=static_cast<uint16_t>(DayOfWeek::FirstDayOfWeek); i<=static_cast<uint16_t>(DayOfWeek::LastDayOfWeek); i++)
    _tprintf(_T("Week Day: %s %s\n"), CHebrewCalendar::GetFullStringDayOfWeek(static_cast<DayOfWeek>(i)).c_str(), CHebrewCalendar::GetAbrStringDayOfWeek(static_cast<DayOfWeek>(i)).c_str());

  hdate1 = CDate<CHebrewCalendar>::CurrentLocalDate();
  s = hdate1.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The current local date (in the Hebrew Calendar) is %s\n"), s.c_str());

  s = CDate<>{2006, static_cast<uint16_t>(Month::March), 22}.AsHebrewCDate().Format(_T("%Y-%m-%d"));

  _tprintf(_T("Moslem days of the week\n"));
  for (auto i= static_cast<uint16_t>(DayOfWeek::FirstDayOfWeek); i<=static_cast<uint16_t>(DayOfWeek::LastDayOfWeek); i++)
    _tprintf(_T("Week Day: %s %s\n"), CMoslemCalendarCivil16::GetFullStringDayOfWeek(static_cast<DayOfWeek>(i)).c_str(), CMoslemCalendarCivil16::GetAbrStringDayOfWeek(static_cast<DayOfWeek>(i)).c_str());

  //test the range of the class
  int32_t StartYear{-2000000};
  int32_t EndYear{2000000};
  int32_t nLastDateValue{0};
  _tprintf(_T("Doing CDate Gregorian calendar algorithm validation\n"));
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  bool bSuccess{CGregorianCalendar::DMYToNum(StartYear - 1, 12, 31, nLastDateValue, false)};
  assert(bSuccess);
  UNREFERENCED_PARAMETER(bSuccess);
  CLLDate<CGregorianCalendar> startTime{CLLDate<CGregorianCalendar>::CurrentSystemTime()};
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=1; wMonth<=12; ++wMonth)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        int32_t nDays{0};
        bSuccess = CGregorianCalendar::DMYToNum(lYear, wMonth, wDay, nDays, false); //NOLINT(clang-analyzer-deadcode.DeadStores)
        DTIME_ASSERT(bSuccess);
        int32_t tYear{0};
        uint16_t tMonth{0};
        uint16_t tDay{0};
        CGregorianCalendar::NumToDMY(nDays, tYear, tMonth, tDay);
        if ((tYear != lYear) || (tMonth != wMonth) || (tDay != wDay))
        {
          _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
          DTIME_ASSERT(false);
        }
        if (nDays != (nLastDateValue + 1))
        {
          _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
          DTIME_ASSERT(false);
        }
        nLastDateValue = nDays;
      }
    }
  }
  double fSecondsTaken{(CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue()};
  _tprintf(_T("CDate Gregorian calendar algorithm time taken: %f seconds\n"), fSecondsTaken);
  int32_t nDays{CDate<CGregorianCalendar>(EndYear - 1, 12, 31) - CDate<CGregorianCalendar>(StartYear, 1, 1)};
  _tprintf(_T("CDate Gregorian calendar algorithm days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Gregorian calendar algorithm speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);


  _tprintf(_T("Doing CDate Gregorian calendar object validation\n"));
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  bSuccess = CGregorianCalendar::DMYToNum(StartYear - 1, 12, 31, nLastDateValue, false);
  assert(bSuccess);
  UNREFERENCED_PARAMETER(bSuccess);
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=1; wMonth<=12; ++wMonth)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        x = CDate<CGregorianCalendar>{lYear, wMonth, wDay};
        if (x.IsValid())
        {
          const DateS ds{x.GetDate()};
          if ((ds.Year != lYear) || (ds.Month != wMonth) || (ds.Day != wDay))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          const int32_t nNewDateValue{x.GetValue()};
          if (nNewDateValue != (nLastDateValue + 1))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          nLastDateValue = nNewDateValue;
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CDate Gregorian calendar object time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, 12, 31} - CDate<CGregorianCalendar>{StartYear, 1, 1};
  _tprintf(_T("CDate Gregorian calendar object days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Gregorian calendar object speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CDate Julian calendar algorithm validation\n"));
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  bSuccess = CJulianCalendar::DMYToNum(StartYear - 1, 12, 31, nLastDateValue, false);
  assert(bSuccess);
  UNREFERENCED_PARAMETER(bSuccess);
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=1; wMonth<=12; ++wMonth)
    {
      const uint16_t wLastDayInMonth{CJulianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay = 1; wDay <= wLastDayInMonth; wDay++)
      {
        nDays = 0;
        bSuccess = CJulianCalendar::DMYToNum(lYear, wMonth, wDay, nDays, false); //NOLINT(clang-analyzer-deadcode.DeadStores)
        DTIME_ASSERT(bSuccess);
        int32_t tYear{0};
        uint16_t tMonth{0};
        uint16_t tDay{0};
        CJulianCalendar::NumToDMY(nDays, tYear, tMonth, tDay);
        if ((tYear != lYear) || (tMonth != wMonth) || (tDay != wDay))
        {
          _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
          DTIME_ASSERT(false);
        }
        if (nDays != (nLastDateValue + 1))
        {
          _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
          DTIME_ASSERT(false);
        }
        nLastDateValue = nDays;
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CDate Julian calendar algorithm time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CJulianCalendar>{EndYear - 1, 12, 31} - CDate<CJulianCalendar>{StartYear, 1, 1};
  _tprintf(_T("CDate Julian calendar algorithm days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Julian calendar algorithm speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CDate Julian calendar object validation\n"));
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  bSuccess = CJulianCalendar::DMYToNum(StartYear - 1, 12, 31, nLastDateValue, false);
  assert(bSuccess);
  UNREFERENCED_PARAMETER(bSuccess);
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=1; wMonth<=12; ++wMonth)
    {
      const uint16_t wLastDayInMonth{CJulianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        p = CDate<CJulianCalendar>{lYear, wMonth, wDay};
        if (p.IsValid())
        {
          const DateS ds{p.GetDate()};
          if ((ds.Year != lYear) || (ds.Month != wMonth) || (ds.Day != wDay))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          const int32_t nNewDateValue{p.GetValue()};
          if (nNewDateValue != (nLastDateValue + 1))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          nLastDateValue = nNewDateValue;
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CDate Julian calendar object time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CJulianCalendar>{EndYear - 1, 12, 31} - CDate<CJulianCalendar>{StartYear, 1, 1};
  _tprintf(_T("CDate Julian calendar object days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Julian calendar object speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CDate Moslem Civil 16 calendar validation\n"));
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  bool bLastDateValueValid = false;
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(MoslemMonth::Muharram); wMonth<=static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja); wMonth++)
    {
      const uint16_t wLastDayInMonth{CMoslemCalendarCivil16::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        mdate4 = CDate<CMoslemCalendarCivil16>{lYear, wMonth, wDay};
        if (mdate4.IsValid())
        {
          const DateS ds{mdate4.GetDate()};
          if ((ds.Year != lYear) || (ds.Month != wMonth) || (ds.Day != wDay))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (mdate4.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = mdate4.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CDate Moslem Civil 16 calendar time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CMoslemCalendarCivil16>{EndYear - 1, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja), CMoslemCalendarCivil16::DaysInMonth(EndYear - 1, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja))} - CDate<CMoslemCalendarCivil16>{StartYear, static_cast<uint16_t>(MoslemMonth::Muharram), 1};
  _tprintf(_T("CDate Moslem Civil 16 calendar days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Moslem Civil 16 calendar speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CDate Moslem Civil 15 calendar validation\n"));
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  CDate<CMoslemCalendarCivil15> mdate1;
  bLastDateValueValid = false;
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(MoslemMonth::Muharram); wMonth<=static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja); wMonth++)
    {
      const uint16_t wLastDayInMonth{CMoslemCalendarCivil15::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        mdate1 = CDate<CMoslemCalendarCivil15>{lYear, wMonth, wDay};
        if (mdate1.IsValid())
        {
          const DateS ds{mdate1.GetDate()};
          if ((ds.Year != lYear) || (ds.Month != wMonth) || (ds.Day != wDay))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (mdate1.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = mdate1.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CDate Moslem Civil 15 calendar time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CMoslemCalendarCivil16>{EndYear - 1, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja), CMoslemCalendarCivil16::DaysInMonth(EndYear - 1, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja))} - CDate<CMoslemCalendarCivil16>{StartYear, static_cast<uint16_t>(MoslemMonth::Muharram), 1};
  _tprintf(_T("CDate Moslem Civil 15 calendar days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Moslem Civil 15 calendar speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CDate Moslem Astronomical 15 calendar validation\n"));
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  CDate<CMoslemCalendarAstronomical15> mdate5;
  bLastDateValueValid = false;
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(MoslemMonth::Muharram); wMonth<=static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja); wMonth++)
    {
      const uint16_t wLastDayInMonth{CMoslemCalendarAstronomical15::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        mdate5 = CDate<CMoslemCalendarAstronomical15>{lYear, wMonth, wDay};
        if (mdate5.IsValid())
        {
          const DateS ds{mdate5.GetDate()};
          if ((ds.Year != lYear) || (ds.Month != wMonth) || (ds.Day != wDay))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (mdate5.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = mdate5.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CDate Moslem Astronomical 15 calendar time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CMoslemCalendarCivil16>{EndYear - 1, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja), CMoslemCalendarCivil16::DaysInMonth(EndYear - 1, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja))} - CDate<CMoslemCalendarCivil16>{StartYear, static_cast<uint16_t>(MoslemMonth::Muharram), 1};
  _tprintf(_T("CDate Moslem Astronomical 15 calendar days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Moslem Astronomical 15 calendar speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CDate Moslem Astronomical 16 calendar validation\n"));
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  CDate<CMoslemCalendarAstronomical16> mdate8;
  bLastDateValueValid = false;
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(MoslemMonth::Muharram); wMonth<=static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja); wMonth++)
    {
      const uint16_t wLastDayInMonth{CMoslemCalendarAstronomical16::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        mdate8 = CDate<CMoslemCalendarAstronomical16>{lYear, wMonth, wDay};
        if (mdate8.IsValid())
        {
          const DateS ds{mdate8.GetDate()};
          if ((ds.Year != lYear) || (ds.Month != wMonth) || (ds.Day != wDay))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (mdate8.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = mdate8.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CDate Moslem Astronomical 16 calendar time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CMoslemCalendarCivil16>{EndYear - 1, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja), CMoslemCalendarCivil16::DaysInMonth(EndYear - 1, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja))} - CDate<CMoslemCalendarCivil16>{StartYear, static_cast<uint16_t>(MoslemMonth::Muharram), 1};
  _tprintf(_T("CDate Moslem Astronomical 16 calendar days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Moslem Astronomical 16 calendar speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CDate Hebrew calendar validation\n"));
  StartYear = -1000000;
  EndYear = 1000000;
  _tprintf(_T("between the years %d and %d\n"), StartYear, EndYear);
  bLastDateValueValid = false;
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    //Note for testing purpose we need to use a different ordering for the Hebrew months, because the New Year occurs in the
    //month Tishri which is logically the 7th Month as opposed the the first month which is Nisan
    for (wMonth=static_cast<uint16_t>(HebrewMonth::Tishrei); wMonth<=CHebrewCalendar::LastMonthOfYear(CHebrewCalendar::IsLeap(lYear)); wMonth++)
    {
      const uint16_t wLastDayInMonth{CHebrewCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        hdate1 = CDate<CHebrewCalendar>{lYear, wMonth, wDay};
        if (hdate1.IsValid())
        {
          const DateS ds{hdate1.GetDate()};
          if ((ds.Year != lYear) || (ds.Month != wMonth) || (ds.Day != wDay))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (hdate1.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = hdate1.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
    for (wMonth=static_cast<uint16_t>(HebrewMonth::Nissan); wMonth<=static_cast<uint16_t>(HebrewMonth::Elul); wMonth++)
    {
      const uint16_t wLastDayInMonth{CHebrewCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay <= wLastDayInMonth; wDay++)
      {
        hdate1 = CDate<CHebrewCalendar>{lYear, wMonth, wDay};
        if (hdate1.IsValid())
        {
          const DateS ds{hdate1.GetDate()};
          if ((ds.Year != lYear) || (ds.Month != wMonth) || (ds.Day != wDay))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (hdate1.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = hdate1.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CDate Hebrew calendar time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CHebrewCalendar>{EndYear - 1, static_cast<uint16_t>(HebrewMonth::Elul), CHebrewCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(HebrewMonth::Elul))} - CDate<CHebrewCalendar>{StartYear, static_cast<uint16_t>(HebrewMonth::Nissan), 1};
  _tprintf(_T("CDate Hebrew calendar days roundtripped: %d\n"), nDays);
  _tprintf(_T("CDate Hebrew calendar speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  //Try out the GetAsTM accessor
  x = CDate<CGregorianCalendar>::Epoch2000();
  tm tms;
  bSuccess = x.GetAsTM(tms);
  UNREFERENCED_PARAMETER(bSuccess);

  //Try out the GetAsSystemTime accessor
#ifdef _WIN32
  bSuccess = x.GetAsSystemTime(st); //NOLINT(clang-analyzer-deadcode.DeadStores)

  //Try out the GetAsFileTime accessor
  bSuccess = x.GetAsFileTime(ft); //NOLINT(clang-analyzer-deadcode.DeadStores)
#endif //#ifdef _WIN32

  // Try out the other operators
  int32_t l{0};
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  l = x.GetValue();
  _tprintf(_T("Today is %d days since the Gregorian epoch\n"), l);
  l = x.JD();
  _tprintf(_T("Today is %d days since the Julian Day epoch\n"), l);
  if (x.IsLeap())
    _tprintf(_T("This year is a leap year\n"));
  else
    _tprintf(_T("This year is not a leap year\n"));
  const uint16_t diy{x.DaysInYear()};
  _tprintf(_T("This year contains %d days \n"), static_cast<int>(diy));
  const uint16_t dim{x.DaysInMonth()};
  _tprintf(_T("This month contains %d days \n"), static_cast<int>(dim));
  x.AddYear(100);
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("100 years from today is %s\n"), s.c_str());
  x.AddYear(-100);
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("100 years from today - 100 years is %s\n"), s.c_str());
  x.AddMonth(100);
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("100 months from today is %s\n"), s.c_str());
  x.AddMonth(-100);
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("100 months from today - 100 months is %s\n"), s.c_str());
  x.AddWeek(100);
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("100 weeks from today is %s\n"), s.c_str());
  x.AddWeek(-100);
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("100 weeks from today - 100 weeks is %s\n"), s.c_str());
  dow = x.GetDayOfWeek();
  _tprintf(_T("Todays day of week is %d\n"), static_cast<int>(dow));
#ifdef _WIN32
  s = x.GetFullStringDayOfWeek();
  _tprintf(_T("Todays full string day of week is %s\n"), s.c_str());
  s = x.GetAbrStringDayOfWeek();
  _tprintf(_T("Todays Abbreviated string day of week is %s\n"), s.c_str());
  s = x.GetFullStringMonth();
  _tprintf(_T("Todays full string Month is %s\n"), s.c_str());
  s = x.GetAbrStringMonth();
  _tprintf(_T("Todays Abbreviated string Month is %s\n"), s.c_str());
#endif //#ifdef _WIN32
  _tprintf(_T("Today is %d days since the 1st of this year\n"), static_cast<int>(x.DaysSinceFirstOfYear()));
  _tprintf(_T("Today is %d days since the 0th of this year\n"), static_cast<int>(x.DaysSinceEndPreviousYear()));

#ifdef _WIN32
  bSuccess = x.GetAsSystemTime(st); //NOLINT(clang-analyzer-deadcode.DeadStores)
#endif //#ifdef _WIN32

  bool IsCE{false};
  const int32_t y{x.GetCEBCEYear(IsCE)};
  UNREFERENCED_PARAMETER(y);

  //Creation of other CDate's from this instance
  x = CDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::February), 3};
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The CDate used to test CDate creation from this instance is %s\n"), s.c_str());
  CDate<CGregorianCalendar> c{x.FirstThisMonth()};
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("FirstThisMonth returns %s\n"), s.c_str());
  c = x.LastThisMonth();
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("LastThisMonth returns %s\n"), s.c_str());
  c = x.FirstThisYear();
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("FirstThisYear returns %s\n"), s.c_str());
  c = x.LastThisYear();
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("LastThisYear returns %s\n"), s.c_str());

  //Assignment operator has been tested before
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  c = x + 100;
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("100 days from today is %s\n"), s.c_str());
  c = c - 100;
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("100 days from today  - 100 days is %s\n"), s.c_str());
  const int32_t diff{CDate<CGregorianCalendar>::CurrentLocalDate() - CDate<CGregorianCalendar>{2000, static_cast<uint16_t>(Month::January), 1}};
  _tprintf(_T("the timespan between today and Epoch 2000 is %d\n"), diff);
  c += 1000;
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("today incremented by 1000 days is %s\n"), s.c_str());
  c -= 1000;
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("today incremented by 1000 days decremented by 1000 is %s\n"), s.c_str());
  ++c;
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Tomorrow is %s\n"), s.c_str());
  --c;
  --c;
  s = c.Format(_T("%Y-%m-%d"));
  _tprintf(_T("Yesterday was %s\n"), s.c_str());

  c = CDate<CGregorianCalendar>::CurrentLocalDate();

  //Full test of the Format function
  _tprintf(_T("A full test of Format for Today gives\n"));
#ifdef _WIN32
  s = c.Format(_T("%a"));
  _tprintf(_T("Abbreviated weekday name: %s\n"), s.c_str());
  s = c.Format(_T("%A"));
  _tprintf(_T("Full weekday name: %s\n"), s.c_str());
  s = c.Format(_T("%b"));
  _tprintf(_T("Abbreviated month name: %s\n"), s.c_str());
  s = c.Format(_T("%B"));
  _tprintf(_T("Full month name: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = c.Format(_T("%d"));
  _tprintf(_T("Day of month as decimal number (01 - 31): %s\n"), s.c_str());
  s = c.Format(_T("%j"));
  _tprintf(_T("Day of year as decimal number (001 - 366): %s\n"), s.c_str());
  s = c.Format(_T("%m"));
  _tprintf(_T("Month as decimal number (01 - 12): %s\n"), s.c_str());
  s = c.Format(_T("%U"));
  _tprintf(_T("Week of year as decimal number: %s\n"), s.c_str());
  s = c.Format(_T("%w"));
  _tprintf(_T("Weekday as decimal number (1 - 7), Sunday is 1: %s\n"), s.c_str());
#ifdef _WIN32
  s = c.Format(_T("%x"));
  _tprintf(_T("Date representation for current locale: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = c.Format(_T("%y"));
  _tprintf(_T("Year without century, as decimal number (00 - 99): %s\n"), s.c_str());
  s = c.Format(_T("%Y"));
  _tprintf(_T("Year with century, as decimal number: %s\n"), s.c_str());
#ifdef _WIN32
  s = c.Format(_T("%#x"));
  _tprintf(_T("Long date representation, appropriate to current locale: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = c.Format(_T("%#d"));
  _tprintf(_T("Day of month without leading zeros: %s\n"), s.c_str());
  s = c.Format(_T("%#j"));
  _tprintf(_T("Day of year without leading zeros: %s\n"), s.c_str());
  s = c.Format(_T("%#m"));
  _tprintf(_T("Month without leading zeroes: %s\n"), s.c_str());
  s = c.Format(_T("%#U"));
  _tprintf(_T("Week of year without leading zeroes: %s\n"), s.c_str());
  s = c.Format(_T("%#y"));
  _tprintf(_T("Year without century without leading zeroes: %s\n"), s.c_str());

  //Try out the week of year function
  x = CDate<CGregorianCalendar>{1997, static_cast<uint16_t>(Month::January), 1};
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The first of January for testing the Week Of Year function is %s\n"), s.c_str());
  //_tprintf(_T("Setting the beginning of the week to Thursday\n"));
  CDate<CGregorianCalendar> v{1997, static_cast<uint16_t>(Month::January), 1, DayOfWeek::Thursday};
  s = v.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The 1st Thursday of 1997 is %s\n"), s.c_str());
  for (uint16_t i=1; i<32; i++)
  {
    const CDate<CGregorianCalendar> p3{1997, static_cast<uint16_t>(Month::January), i};
    const uint16_t woy{p3.GetWeekOfYear()};
    s = p3.Format(_T("%Y-%m-%d"));
    _tprintf(_T("The week of the year for %s is %d\n"), s.c_str(), static_cast<int>(woy));
  }

  //Try out the week of year function
  x = CDate<CGregorianCalendar>{1997, static_cast<uint16_t>(Month::January), 1};
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The first of January for testing the Week Of year function is %s\n"), s.c_str());
  v = CDate<CGregorianCalendar>{1997, static_cast<uint16_t>(Month::January), 1, DayOfWeek::Monday};
  s = v.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The 1st Monday of 1997 is %s\n"), s.c_str());
  for (uint16_t i=1; i<32; i++)
  {
    const CDate<CGregorianCalendar> p3{x.GetYear(), x.GetMonth(), i};
    const uint16_t woy{p3.GetWeekOfYear()};
    s = p3.Format(_T("%Y-%m-%d"));
    _tprintf(_T("The week of the year for %s is %d\n"), s.c_str(), static_cast<int>(woy));
  }

  //Try out the week of month function
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  x = x.FirstThisMonth();
  s = x.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The first of this month for testing the Week Of month function is %s\n"), s.c_str());
  v = CDate<CGregorianCalendar>{x.GetYear(), x.GetMonth(), 1, DayOfWeek::Thursday};
  s = v.Format(_T("%Y-%m-%d"));
  _tprintf(_T("The 1st Thursday of the month is %s\n"), s.c_str());
  for (uint16_t i=1; i<=x.DaysInMonth(); i++)
  {
    const CDate<CGregorianCalendar> p3{x.GetYear(), x.GetMonth(), i};
    const uint16_t wom{p3.GetWeekOfMonth()};
    s = p3.Format(_T("%Y-%m-%d"));
    _tprintf(_T("The week of the month for %s is %d\n"), s.c_str(), static_cast<int>(wom));
  }

  //try out the collate function
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  const int32_t coll{x.Collate()};
  _tprintf(_T("The collate function for today returns %d\n"), coll);
  x.SetFromCollate(coll);
  _tprintf(_T("Using SetFromCollate for today created %s\n"), x.Format(_T("%Y-%m-%d")).c_str());

  //try out the equality operators
  x = CDate<CGregorianCalendar>::CurrentSystemDate();
  CDate<CGregorianCalendar> y2{x};
  DTIME_ASSERT(x == y2);
  DTIME_ASSERT(!(x != y2));
  y2 += 1;
  DTIME_ASSERT(y2 != x);
  DTIME_ASSERT(y2 >= x);
  DTIME_ASSERT(x <= y2);
  DTIME_ASSERT(x < y2);

  mdate1 = CDate<CMoslemCalendarCivil15>{1421, static_cast<uint16_t>(MoslemMonth::Muharram), 1};
  s = mdate1.Format(_T("%Y-%m-%d"));
  const CDate<CJulianCalendar> mdate2{mdate1.AsJulianCDate()};
  s = mdate2.Format(_T("%Y-%m-%d"));
  const CDate<CGregorianCalendar> mdate3{mdate1.AsGregorianCDate()};
  s = mdate3.Format(_T("%Y-%m-%d"));

  mdate1 = CDate<CMoslemCalendarCivil15>{1427, static_cast<uint16_t>(MoslemMonth::Dhu_l_Hijja), 28};
  DateS ds{mdate1.GetDate()};
  ds = mdate1.AsGregorianCDate().GetDate();
  ds = mdate1.AsJulianCDate().GetDate();

  p = CDate<CJulianCalendar>{622, static_cast<uint16_t>(Month::July), 16};
  mdate1 = CDate<CMoslemCalendarCivil15>{1, 1, 1};
  s = mdate1.Format(_T("%Y-%m-%d"));
  ds = mdate1.AsJulianCDate().GetDate();

  const CDate<CMoslemCalendarCivil16> mdate6{16, 6, 5};
  s = mdate6.Format(_T("%Y-%m-%d"));
  ds = mdate6.AsJulianCDate().GetDate();

  mdate1 = CDate<CMoslemCalendarCivil15>{16, 6, 5};
  s = mdate1.Format(_T("%Y-%m-%d"));
  ds = mdate1.AsJulianCDate().GetDate();

  const CDate<CMoslemCalendarAstronomical16> mdate7{16, 6, 5};
  s = mdate7.Format(_T("%Y-%m-%d"));
  ds = mdate7.AsJulianCDate().GetDate();

  const CDate<CMoslemCalendarAstronomical16> mdate9{16, 6, 5};
  s = mdate9.Format(_T("%Y-%m-%d"));
  ds = mdate9.AsJulianCDate().GetDate();

  ds = mdate1.GetDate();
  ds = mdate1.AsGregorianCDate().GetDate();
  ds = mdate1.AsJulianCDate().GetDate();

  mdate1 = CDate<CMoslemCalendarCivil15>::CurrentLocalDate();
  _tprintf(_T("A full test of Format for Today in the Moslem Civil 15 calendar gives\n"));
#ifdef _WIN32
  s = mdate1.Format(_T("%a"));
  _tprintf(_T("Abbreviated weekday name: %s\n"), s.c_str());
  s = mdate1.Format(_T("%A"));
  _tprintf(_T("Full weekday name: %s\n"), s.c_str());
  s = mdate1.Format(_T("%b"));
  _tprintf(_T("Abbreviated month name: %s\n"), s.c_str());
  s = mdate1.Format(_T("%B"));
  _tprintf(_T("Full month name: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = mdate1.Format(_T("%d"));
  _tprintf(_T("Day of month as decimal number (01 - 31): %s\n"), s.c_str());
  s = mdate1.Format(_T("%j"));
  _tprintf(_T("Day of year as decimal number (001 - 355): %s\n"), s.c_str());
  s = mdate1.Format(_T("%m"));
  _tprintf(_T("Month as decimal number (01 - 12): %s\n"), s.c_str());
  s = mdate1.Format(_T("%U"));
  _tprintf(_T("Week of year as decimal number: %s\n"), s.c_str());
  s = mdate1.Format(_T("%w"));
  _tprintf(_T("Weekday as decimal number (1 - 7), Sunday is 1: %s\n"), s.c_str());
#ifdef _WIN32
  s = mdate1.Format(_T("%x"));
  _tprintf(_T("Date representation for current locale: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = mdate1.Format(_T("%y"));
  _tprintf(_T("Year without century, as decimal number (00 - 99): %s\n"), s.c_str());
  s = mdate1.Format(_T("%Y"));
  _tprintf(_T("Year with century, as decimal number: %s\n"), s.c_str());
#ifdef _WIN32
  s = mdate1.Format(_T("%#x"));
  _tprintf(_T("Long date representation, appropriate to current locale: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = mdate1.Format(_T("%#d"));
  _tprintf(_T("Day of month without leading zeros: %s\n"), s.c_str());
  s = mdate1.Format(_T("%#j"));
  _tprintf(_T("Day of year without leading zeros: %s\n"), s.c_str());
  s = mdate1.Format(_T("%#m"));
  _tprintf(_T("Month without leading zeroes: %s\n"), s.c_str());
  s = mdate1.Format(_T("%#U"));
  _tprintf(_T("Week of year without leading zeroes: %s\n"), s.c_str());
  s = mdate1.Format(_T("%#y"));
  _tprintf(_T("Year without century without leading zeroes: %s\n"), s.c_str());


  //Test of the CLTimeSpan class

  //first the constructors
  CLTimeSpan ts;
  DTIME_ASSERT(!ts.IsValid());

  ts = CLTimeSpan{3, 4, 5, 6};
  int32_t dd{ts.GetTotalDays()};
  UNREFERENCED_PARAMETER(dd);
  uint16_t hh{ts.GetHours()};
  UNREFERENCED_PARAMETER(hh);
  uint16_t mm{ts.GetMinutes()};
  UNREFERENCED_PARAMETER(mm);
  uint16_t ss{ts.GetSeconds()};
  UNREFERENCED_PARAMETER(ss);
  s = ts.Format(_T("%D %H:%M:%S"));

  ts = CLTimeSpan{3, 27, 59, 69};
  s = ts.Format(_T("%D %H:%M:%S"));

  ts = CLTimeSpan{-3, 4, 5, 6};
  s = ts.Format(_T("%D %H:%M:%S"));

  CLTimeSpan ts2{ts};
  s = ts2.Format(_T("%D %H:%M:%S"));
#ifdef _WIN32
  s = ts2.Format(CLTimeSpan::GetWindowsFormatString());
#endif //#ifdef _WIN32


#ifdef _AFX
  const CTimeSpan OneDay{1, 0, 0, 0};
  ts = CLTimeSpan{OneDay};
  s = ts.Format(_T("%D %H:%M:%S"));
#endif //#ifdef _AFX

  ts = CLTimeSpan{86400};
  s = ts.Format(_T("%D %H:%M:%S"));

  ts = CLTimeSpan{-23};
  dd = ts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = ts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = ts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss = ts.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = ts.Format(_T("%D %H:%M:%S"));
  double secs{ts.SecondsAsDouble()};
  UNREFERENCED_PARAMETER(secs);

  ts = CLTimeSpan{-86401};
  dd = ts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = ts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = ts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss = ts.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = ts.Format(_T("%D %H:%M:%S"));
  secs = ts.SecondsAsDouble(); //NOLINT(clang-analyzer-deadcode.DeadStores)

  ts = CLTimeSpan{-86401};
  dd = ts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = ts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = ts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss = ts.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = ts.Format(_T("%D %H:%M:%S"));
  secs = ts.SecondsAsDouble(); //NOLINT(clang-analyzer-deadcode.DeadStores)

  ts = CLTimeSpan{86399};
  s = ts.Format(_T("%D %H:%M:%S"));
  secs = ts.SecondsAsDouble(); //NOLINT(clang-analyzer-deadcode.DeadStores)

#ifdef _AFX
  //COleDateTimeSpan Construction
  const COleDateTimeSpan olets{19, 12, 10, 3};
  ts = CLTimeSpan{olets};
  s = ts.Format(_T("%D %H:%M:%S"));
#endif //#ifdef _AFX

  //The int64_t constructor
  ts = CLTimeSpan{3155760000};
  dd = ts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = ts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = ts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss = ts.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = ts.Format(_T("%D %H:%M:%S"));

  //Set operators have been tested via the constructors

  //static Constructors

  ts = CLTimeSpan::OneDay();
  s = ts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("One Day as a time span is %s\n"), s.c_str());
  ts = CLTimeSpan::OneHour();
  s = ts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("One Hour as a time span is %s\n"), s.c_str());
  ts = CLTimeSpan::OneMinute();
  s = ts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("One Minute as a time span is %s\n"), s.c_str());
  ts = CLTimeSpan::OneSecond();
  s = ts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("One Second as a time span is %s\n"), s.c_str());

  //days, Hours, Minutes & seconds accessors
  //already tested via Format

  //Overloaded Arithmetic Operators
  CLTimeSpan ts1{CLTimeSpan{365, 0, 0, 0}};
  ts2 = CLTimeSpan::OneDay();
  //operator= already tested
  CLTimeSpan ts3{ts1 + ts2};
  _tprintf(_T("365 days + One day is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts1 - ts2;
  _tprintf(_T("365 days - One day is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts2 - ts1;

  _tprintf(_T("One day - One year is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts2 = CLTimeSpan::OneDay();
  ts1 += ts2;
  _tprintf(_T("365 days incremented by one day is %s\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  ts1 -= ts2;
  _tprintf(_T("365 days + one day decremented by one day is %s\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts2 -= ts1;
  _tprintf(_T("One day decremented by 365 days is %s\n"), ts2.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts1.Negate();
  ts2 = CLTimeSpan::OneDay();
  ts3 = ts1 - ts2;
  _tprintf(_T("-365 days - one day is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts1 + ts2;
  _tprintf(_T("-365 days + one day is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts2 = CLTimeSpan::OneDay();
  ts2.Negate();
  ts3 = ts1 - ts2;
  _tprintf(_T("365 days - (-one day) is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts1 + ts2;
  _tprintf(_T("365 days + (-one day) is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts1.Negate();
  ts2 = CLTimeSpan::OneDay();
  ts1 += ts2;
  _tprintf(_T("-365 days incremented by one day is %s\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  ts1 -= ts2;
  _tprintf(_T("-365 days + 1 day decremented by one day is %s\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  ts2 = CLTimeSpan{365, 0, 0, 0};
  ts2.Negate();
  ts1 -= ts2;
  _tprintf(_T("-365 days decremented by -one year is %s\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  ts1 += ts2;
  _tprintf(_T("-365 days - -365 days incremented by -365 days is %s\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts2 = CLTimeSpan::OneDay();
  ts2.Negate();
  ts3 = ts1 - ts2;
  _tprintf(_T("365 days - -one day is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts1 + ts2;
  _tprintf(_T("365 days + -one day is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts2 - ts1;
  _tprintf(_T("-One day - 365 days is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts2 + ts1;
  _tprintf(_T("-One day + 365 days is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts2 = CLTimeSpan::OneDay();
  ts3 = ts1 * 100;

  dd = ts3.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = ts3.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = ts3.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss = ts3.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = ts3.Format(_T("%D %H:%M:%S"));

  _tprintf(_T("365 days * 100 is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = 100 * ts1;
  _tprintf(_T("100 * 365 days is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts1 / 3;
  _tprintf(_T("365 days / 3 is %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts1 *= 100;
  _tprintf(_T("365 days *= 100 is %s\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts1 /= 100;
  _tprintf(_T("365 days /= 100 is %s\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  ts1 = CLTimeSpan{365, 6, 10, 34};
  ts3 = ts1 * 100;
  _tprintf(_T("%s * 100 is"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  _tprintf(_T(" %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());
  ts3 = ts1 / 100;
  _tprintf(_T("%s / 100 is"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  _tprintf(_T(" %s\n"), ts3.Format(_T("%D %H:%M:%S")).c_str());

  s = ts1.Format(_T("%D %H:%M:%S"));
  ts3 = - ts1;
  s = ts3.Format(_T("%D %H:%M:%S"));

  //overloaded equality operators
  ts1 = CLTimeSpan{365, 0, 0, 0};
  ts2 = CLTimeSpan::OneDay();
  DTIME_ASSERT(ts1 >= ts2);
  DTIME_ASSERT(ts1 > ts2);
  DTIME_ASSERT(ts2 < ts1);
  DTIME_ASSERT(ts2 <= ts1);
  DTIME_ASSERT(ts2 != ts1);
  DTIME_ASSERT(!(ts2 == ts1));

  //Full test of the Format function
  ts1 = CLTimeSpan{1, 2, 3, 4};
  _tprintf(_T("A full test of CLTimeSpan::Format for %s gives\n"), ts1.Format(_T("%D %H:%M:%S")).c_str());
  s = ts1.Format(_T("%D"));
  _tprintf(_T("Days in the timespan: %s\n"), s.c_str());
  s = ts1.Format(_T("%H"));
  _tprintf(_T("Hours in the timespan: %s\n"), s.c_str());
  s = ts1.Format(_T("%M"));
  _tprintf(_T("Minutes in the timespan: %s\n"), s.c_str());
  s = ts1.Format(_T("%S"));
  _tprintf(_T("Seconds in the timespan: %s\n"), s.c_str());
  s = ts1.Format(_T("%#H"));
  _tprintf(_T("Hours in the timespan without leading zeros: %s\n"), s.c_str());
  s = ts1.Format(_T("%#M"));
  _tprintf(_T("Minutes in the timespan without leading zeros: %s\n"), s.c_str());
  s = ts1.Format(_T("%#S"));
  _tprintf(_T("Seconds in the timespan without leading zeros: %s\n"), s.c_str());


  //Try out of the CLTimeOfDay class

  //first the constructors
  CLTimeOfDay<> tod;
  DTIME_ASSERT(!tod.IsValid());

  //tod = CLTimeOfDay<>{23, 59, 61, true}; //should assert
  //tod = CLTimeOfDay<>{24, 59, 61, true}; //should assert
  //tod = CLTimeOfDay<>{23, 59, 69, true}; //should assert

#ifdef _WIN32
  SYSTEMTIME stnow{};
  GetLocalTime(&stnow);
  tod = CLTimeOfDay<>(stnow);
  s = tod.Format(_T("%H:%M:%S"));
  s = tod.Format(CLTimeOfDay<>::GetWindowsFormatString());
#endif //#ifdef _WIN32

  CLTimeOfDay<> tod2{tod};
  s = tod.Format(_T("%H:%M:%S"));

  //Set functions tested via the constructors

  //The static constructors
  tod = CLTimeOfDay<>::CurrentLocalTime();
  s = tod.Format(_T("%H:%M:%S"));
  _tprintf(_T("The local time of day is %s\n"), s.c_str());

  tod = CLTimeOfDay<>::CurrentSystemTime();
  s = tod.Format(_T("%H:%M:%S"));
  _tprintf(_T("The UTC time of day is %s\n"), s.c_str());

  tod = CLTimeOfDay<>::Midnight();
  s = tod.Format(_T("%H:%M:%S"));
  _tprintf(_T("Midnight as a time of day is %s\n"), s.c_str());

  tod = CLTimeOfDay<>::Midday();
  s = tod.Format(_T("%H:%M:%S"));
  _tprintf(_T("Midday as a time of day is %s\n"), s.c_str());

  //try out the operations
  tod = CLTimeOfDay<>::CurrentSystemTime();
  uint16_t wVal{tod.GetHours()}; //NOLINT(clang-analyzer-deadcode.DeadStores)
  wVal = tod.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  const auto lVal{tod.GetTotalSeconds()};
  UNREFERENCED_PARAMETER(lVal);
  wVal = tod.GetSeconds();
  UNREFERENCED_PARAMETER(wVal);
  DTIME_ASSERT(tod.IsValid());
  const uint32_t col{tod.Collate()};
  _tprintf(_T("The collate function for the Current UTC time of day is\n"));
  _tprintf(_T("%u\n"), col);
#ifdef _WIN32
  bSuccess = tod.GetAsSystemTime(stnow); //NOLINT(clang-analyzer-deadcode.DeadStores)
#endif //#ifdef _WIN32

  //Try out the GetAsTM accessor
  tod = CLTimeOfDay<>::CurrentSystemTime();
  bSuccess = tod.GetAsTM(tms); //NOLINT(clang-analyzer-deadcode.DeadStores)

  //Arithmetic operators

  CLTimeOfDay<> tod3{12, 1, 2};
  CLTimeSpan  ts4{1, 25, 59, 45};
  //operator= already tested
  CLTimeOfDay<> tod4{tod3 + ts4};
  s = tod4.Format(_T("%H:%M:%S"));
  String s2{tod3.Format(_T("%H:%M:%S"))};
  String s3{ts4.Format(_T("%D %H:%M:%S"))};
  _tprintf(_T("%s"), s2.c_str());
  _tprintf(_T(" + %s = "), s3.c_str());
  _tprintf(_T("%s\n"), s.c_str());

  tod2 = tod3 - ts4;
  s2 = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  s3 = ts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" - %s = "), s3.c_str());
  s = tod2.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  s2 = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  tod3 += ts4;
  s3 = ts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" += %s = "), s3.c_str());
  s = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  s2 = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  tod3 -= ts4;
  s3 = ts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" -= %s = "), s3.c_str());
  s = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  //try out negative timespans
  ts4.Negate();
  tod4 = tod3 + ts4;
  s2 = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  s3 = ts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" + %s = "), s3.c_str());
  s = tod4.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  tod2 = tod3 - ts4;
  s2 = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  s3 = ts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" - %s = "), s3.c_str());
  s = tod2.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  s2 = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  tod3 += ts4;
  s3 = ts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" += %s = "), s3.c_str());
  s = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  s2 = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  tod3 -= ts4;
  s3 = ts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" -= %s = "), s3.c_str());
  s = tod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  //equality operators

  CLTimeOfDay<> tod1{12, 1, 2};
  tod2 = CLTimeOfDay<>{13, 0, 2};

  DTIME_ASSERT(tod1 <= tod2);
  DTIME_ASSERT(tod1 < tod2);
  DTIME_ASSERT(tod2 > tod1);
  DTIME_ASSERT(tod2 >= tod1);
  DTIME_ASSERT(tod2 != tod1);
  DTIME_ASSERT(!(tod2 == tod1));

  //Full test of the Format function
  tod1 = CLTimeOfDay<>::CurrentLocalTime();
  s = tod1.Format(_T("%H:%M:%S"));
  _tprintf(_T("A full test of CLTimeOfDay<>::Format for Now (LOCAL) gives\n"));
  s = tod1.Format(_T("%H"));
  _tprintf(_T("Hours in the current day: %s\n"), s.c_str());
  s = tod1.Format(_T("%h"));
  _tprintf(_T("AM / PM Hours in the current day: %s\n"), s.c_str());
#ifdef _WIN32
  s = tod1.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the current day: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = tod1.Format(_T("%M"));
  _tprintf(_T("Minutes in the current day: %s\n"), s.c_str());
  s = tod1.Format(_T("%S"));
  _tprintf(_T("Seconds in the current day: %s\n"), s.c_str());
  s = tod1.Format(_T("%#H"));
  _tprintf(_T("Hours in the current day without leading zeros: %s\n"), s.c_str());
  s = tod1.Format(_T("%#h"));
  _tprintf(_T("AM / PM Hours in the current day without leading zeros: %s\n"), s.c_str());
  s = tod1.Format(_T("%#M"));
  _tprintf(_T("Minutes in the current day without leading zeros: %s\n"), s.c_str());
  s = tod1.Format(_T("%#S"));
  _tprintf(_T("Seconds in the current day without leading zeros: %s\n"), s.c_str());
  tod2 = CLTimeOfDay<>{11, 1, 2};
#ifdef _WIN32
  s = tod2.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the 11:01:02: %s\n"), s.c_str());
  tod2 = CLTimeOfDay<>{12, 1, 2};
  s = tod2.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the 12:01:02: %s\n"), s.c_str());
  tod2 = CLTimeOfDay<>{13, 1, 2};
  s = tod2.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the 13:01:02: %s\n"), s.c_str());
#endif //#ifdef _WIN32

  //Try out the CLDate class
  _tprintf(_T("Doing CLDate Gregorian calendar validation Phase 1 (Using 00:00:00 time of day)\n"));
  StartYear = -2000000;
  EndYear = 2000000;
  int64_t nLastLDateValue{0};
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  bLastDateValueValid = false;
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(Month::January); wMonth<=static_cast<uint16_t>(Month::December); wMonth++)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        const CLDate<CGregorianCalendar> ldate{lYear, wMonth, wDay, 0, 0, 0};
        if (ldate.IsValid())
        {
          const DateLS lds{ldate.GetDate()};
          if ((lds.Year != lYear) || (lds.Month != wMonth) || (lds.Day != wDay) || (lds.Hour != 0) || (lds.Minute != 0) || (lds.Second != 0))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d 00:00:00\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (ldate.GetValue() != (nLastLDateValue + 86400))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d 00:00:00\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastLDateValue = ldate.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CLDate Gregorian calendar validation Phase 1 time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, static_cast<uint16_t>(Month::December), CGregorianCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(Month::December))} - CDate<CGregorianCalendar>{StartYear, static_cast<uint16_t>(Month::January), 1};
  _tprintf(_T("CLDate Gregorian calendar validation Phase 1 days roundtripped: %d\n"), nDays);
  _tprintf(_T("CLDate Gregorian calendar validation Phase 1 speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CLDate Gregorian calendar validation Phase 2 (Using 01:02:03 time of day)\n"));
  nLastLDateValue = 0;
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  bLastDateValueValid = false;
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(Month::January); wMonth<=static_cast<uint16_t>(Month::December); wMonth++)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        const CLDate<CGregorianCalendar> ldate{lYear, wMonth, wDay, 1, 2, 3};
        if (ldate.IsValid())
        {
          const DateLS lds{ldate.GetDate()};
          if ((lds.Year != lYear) || (lds.Month != wMonth) || (lds.Day != wDay) || (lds.Hour != 1) || (lds.Minute != 2) || (lds.Second != 3))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d 01:02:03\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (ldate.GetValue() != (nLastLDateValue + 86400))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d 01:02:03\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastLDateValue = ldate.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CLDate Gregorian calendar validation Phase 2 time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, static_cast<uint16_t>(Month::December), CGregorianCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(Month::December))} - CDate<CGregorianCalendar>{StartYear, static_cast<uint16_t>(Month::January), 1};
  _tprintf(_T("CLDate Gregorian calendar validation Phase 2 days roundtripped: %d\n"), nDays);
  _tprintf(_T("CLDate Gregorian calendar validation Phase 2 speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CLDate Gregorian calendar validation Phase 3 (Using 23:59:59 time of day)\n"));
  nLastLDateValue = 0;
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  bLastDateValueValid = false;
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(Month::January); wMonth<=static_cast<uint16_t>(Month::December); wMonth++)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        const CLDate<CGregorianCalendar> ldate{lYear, wMonth, wDay, 23, 59, 59};
        if (ldate.IsValid())
        {
          const DateLS lds{ldate.GetDate()};
          if ((lds.Year != lYear) || (lds.Month != wMonth) || (lds.Day != wDay) || (lds.Hour != 23) || (lds.Minute != 59) || (lds.Second != 59))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d 23:59:59\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (ldate.GetValue() != (nLastLDateValue + 86400))
            {
              _tprintf(_T("Error occurred testing %d/%d/%d 23:59:59\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastLDateValue = ldate.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CLDate Gregorian calendar validation Phase 3 time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, static_cast<uint16_t>(Month::December), CGregorianCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(Month::December))} - CDate<CGregorianCalendar>{StartYear, static_cast<uint16_t>(Month::January), 1};
  _tprintf(_T("CLDate Gregorian calendar validation Phase 3 days roundtripped: %d\n"), nDays);
  _tprintf(_T("CLDate Gregorian calendar validation Phase 3 speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CLDate Gregorian calendar validation Phase 4 (Using random time of day)\n"));
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(Month::January); wMonth<=static_cast<uint16_t>(Month::December); wMonth++)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
#ifdef _MSC_VER
#pragma warning(suppress: 26472)
#endif //#ifdef _MSC_VER
        const uint16_t nRandomHour{static_cast<uint16_t>(rand() % 24)};
#ifdef _MSC_VER
#pragma warning(suppress: 26472)
#endif //#ifdef _MSC_VER
        const uint16_t nRandomMinute{static_cast<uint16_t>(rand() % 60)};
#ifdef _MSC_VER
#pragma warning(suppress: 26472)
#endif //#ifdef _MSC_VER
        const uint16_t nRandomSecond{static_cast<uint16_t>(rand() % 60)};
        const CLDate<CGregorianCalendar> ldate{lYear, wMonth, wDay, nRandomHour, nRandomMinute, nRandomSecond};
        if (ldate.IsValid())
        {
          const DateLS lds{ldate.GetDate()};
          if ((lds.Year != lYear) || (lds.Month != wMonth) || (lds.Day != wDay) || (lds.Hour != nRandomHour) || (lds.Minute != nRandomMinute) || (lds.Second != nRandomSecond))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d %02d:%02d:%02d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay), static_cast<int>(nRandomHour), static_cast<int>(nRandomMinute), static_cast<int>(nRandomSecond));
            DTIME_ASSERT(false);
          }
          bLastDateValueValid = true; //NOLINT(clang-analyzer-deadcode.DeadStores)
          nLastLDateValue = ldate.GetValue(); //NOLINT(clang-analyzer-deadcode.DeadStores)
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CLDate Gregorian calendar validation Phase 4 time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, static_cast<uint16_t>(Month::December), CGregorianCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(Month::December))} - CDate<CGregorianCalendar>{StartYear, static_cast<uint16_t>(Month::January), 1};
  _tprintf(_T("CLDate Gregorian calendar validation Phase 4 days roundtripped: %d\n"), nDays);
  _tprintf(_T("CLDate Gregorian calendar validation Phase 4 speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);


  //first the constructors
  CLDate<CGregorianCalendar> xlneg{3, static_cast<uint16_t>(Month::January), 2, 0, 0, 0};
  CDate<CGregorianCalendar> ld{xlneg.GetCDate()};
  s = ld.Format(_T("%Y-%m-%d"));
  const CLTimeOfDay<> ltod5{xlneg.GetCLTimeOfDay()};
  UNREFERENCED_PARAMETER(ltod5);
  String sneg{xlneg.Format(_T("%Y-%m-%d %H:%M:%S"))};
  xlneg = CLDate<CGregorianCalendar>{3, static_cast<uint16_t>(Month::January), 2, 0, 0, 0};
  sneg = xlneg.Format(_T("%Y-%m-%d %H:%M:%S"));
  xlneg = CLDate<CGregorianCalendar>{3, static_cast<uint16_t>(Month::January), 2, 4, 3, 2};
  sneg = xlneg.Format(_T("%Y-%m-%d %H:%M:%S"));
  xlneg = CLDate<CGregorianCalendar>{3, static_cast<uint16_t>(Month::January), 2, 23, 59, 59};
  sneg = xlneg.Format(_T("%Y-%m-%d %H:%M:%S"));

  CLDate<CGregorianCalendar> ldate;
  DTIME_ASSERT(!ldate.IsValid());
  ldate = CLDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::November), 3, 12, 11, 10};
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
#ifdef _WIN32
  GetLocalTime(&st);
  ldate = CLDate<CGregorianCalendar>(st);
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  s = ldate.Format(CLDate<>::GetWindowsFormatString());
#endif //#ifdef _WIN32
  ldate = CLDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::March), 5, DayOfWeek::Sunday, 0, 0, 0};
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  CLDate<CJulianCalendar> ldate2{CLDate<CJulianCalendar>{1000, DateEpoch::EPOCH_JD, 0, 0, 0}};
  s = ldate2.Format(_T("%Y-%m-%d %H:%M:%S"));

  const CDate <CGregorianCalendar> x3{1580, 1, 1};
  s = x3.Format(_T("%Y-%m-%d"));
  const CDate <CGregorianCalendar> x4{1581, 1, 1};
  s = x4.Format(_T("%Y-%m-%d"));
  CLDate<CGregorianCalendar> ldate5{1580, 1, 1, 1, 0, 0};
  s = ldate5.Format(_T("%Y-%m-%d %H:%M:%S"));
  ldate5.AddYear(1);
  s = ldate5.Format(_T("%Y-%m-%d %H:%M:%S"));

  //Copy constructor already tested
#ifdef _AFX
  CTime ct{CTime::GetCurrentTime()};
  ldate = CLDate<CGregorianCalendar>{ct};
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  ldate = CLDate<CGregorianCalendar>{CDate<CGregorianCalendar>::CurrentLocalDate(), CLTimeOfDay<>::CurrentLocalTime()};
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
#endif //#ifdef _AFX

#ifdef _WIN32
  //Construction using the Win32 FILETIME structure
  GetLocalTime(&st);
  SystemTimeToFileTime(&st, &ft);
  ldate = CLDate<CGregorianCalendar>{ft};
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
#endif //#ifdef _WIN32

#ifdef _AFX
  COleDateTime ldt{1992, static_cast<uint16_t>(Month::October), 10, 3, 4, 5};
  ldate = CLDate<CGregorianCalendar>{ldt};
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  ldt = COleDateTime{1970, static_cast<uint16_t>(Month::February), 1, 5, 6, 7};
  ldate = CLDate<CGregorianCalendar>{ldt};
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
#endif //#ifdef _AFX

  //Set operators tested via the constructors above

  //Static Constructors
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));

  //Try out the GetAsTM accessor
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  bSuccess = ldate.GetAsTM(tms); //NOLINT(clang-analyzer-deadcode.DeadStores)

  //Need to try out the following functions
  //GetCDate & GetCLTimeOfDay tested through the Format function
#ifdef _WIN32
  bSuccess = ldate.GetAsSystemTime(st); //NOLINT(clang-analyzer-deadcode.DeadStores)
  bSuccess = ldate.GetAsFileTime(ft); //NOLINT(clang-analyzer-deadcode.DeadStores)
#endif //#ifdef _WIN32
#ifdef _AFX
  bSuccess = ldate.GetAsCOleDateTime(dt); //NOLINT(clang-analyzer-deadcode.DeadStores)
#endif //#ifdef _AFX

  ldate.AddYear(10);
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("10 years from now UTC is %s\n"), s.c_str());

  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  ldate.AddWeek(-10);
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("10 weeks before today UTC is %s\n"), s.c_str());

  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  ldate.AddMonth(-14);
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("14 months before today UTC is %s\n"), s.c_str());

  DTIME_ASSERT(ldate.IsValid());
  ldate = CLDate<CGregorianCalendar>{};
  DTIME_ASSERT(!ldate.IsValid());

  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("The current UTC date is %s\n"), s.c_str());
  ldate = CLDate<CGregorianCalendar>::CurrentLocalTime();
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("The current local date is %s\n"), s.c_str());

  //operator= already tried out

  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  ts = CLTimeSpan{10, 10, 10, 10};
  ldate += ts;
  s = ts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("%s from today UTC is "), s.c_str());
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  //Arithmetic operators
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  CLDate<CGregorianCalendar> ldate3{ldate};
  s = ldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  ldate3 += CLTimeSpan{0, 1, 0, 0};
  s = ldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  ldate3 -= CLTimeSpan{0, 1, 0, 0};
  s = ldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  ts = ldate3 - ldate;
  s = ldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  ++ldate3;
  s = ldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  --ldate3;
  s = ldate3.Format(_T("%Y-%m-%d %H:%M:%S"));

  //try out the equality operators
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  CLDate<CGregorianCalendar> ldate4{ldate};
  DTIME_ASSERT(ldate == ldate4);
  DTIME_ASSERT(!(ldate != ldate4));
  ldate4 += CLTimeSpan{0, 0, 0, 1};
  DTIME_ASSERT(ldate4 != ldate);
  DTIME_ASSERT(ldate4 >= ldate);
  DTIME_ASSERT(ldate <= ldate4);
  DTIME_ASSERT(ldate < ldate4);

  ldate = CLDate<CGregorianCalendar>::CurrentLocalTime();
  sString = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  ldate3 = ldate.AsGregorianCLDate();
  s = ldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  ldate2 = ldate.AsJulianCLDate();
  s = ldate2.Format(_T("%Y-%m-%d %H:%M:%S"));

  //Full test of the Format function
  ldate = CLDate<CGregorianCalendar>::CurrentLocalTime();
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("A full test of CLDate::Format for Now (LOCAL) gives\n"));
#ifdef _WIN32
  s = ldate.Format(_T("%a"));
  _tprintf(_T("Abbreviated weekday name: %s\n"), s.c_str());
  s = ldate.Format(_T("%A"));
  _tprintf(_T("Full weekday name: %s\n"), s.c_str());
  s = ldate.Format(_T("%b"));
  _tprintf(_T("Abbreviated month name: %s\n"), s.c_str());
  s = ldate.Format(_T("%B"));
  _tprintf(_T("Full month name: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = ldate.Format(_T("%d"));
  _tprintf(_T("Day of month as decimal number (01 - 31): %s\n"), s.c_str());
  s = ldate.Format(_T("%j"));
  _tprintf(_T("Day of year as decimal number (001 - 366): %s\n"), s.c_str());
  s = ldate.Format(_T("%m"));
  _tprintf(_T("Month as decimal number (01 - 12): %s\n"), s.c_str());
  s = ldate.Format(_T("%U"));
  _tprintf(_T("Week of year as decimal number: %s\n"), s.c_str());
  s = ldate.Format(_T("%w"));
  _tprintf(_T("Weekday as decimal number (1 - 7), Sunday is 1: %s\n"), s.c_str());
#ifdef _WIN32
  s = ldate.Format(_T("%x"));
  _tprintf(_T("Date representation for current locale: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = ldate.Format(_T("%y"));
  _tprintf(_T("Year without century, as decimal number (00 - 99): %s\n"), s.c_str());
  s = ldate.Format(_T("%Y"));
  _tprintf(_T("Year with century, as decimal number: %s\n"), s.c_str());
#ifdef _WIN32
  s = ldate.Format(_T("%#x"));
  _tprintf(_T("Long date representation, appropriate to current locale: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = ldate.Format(_T("%#d"));
  _tprintf(_T("Day of month without leading zeros: %s\n"), s.c_str());
  s = ldate.Format(_T("%#j"));
  _tprintf(_T("Day of year without leading zeros: %s\n"), s.c_str());
  s = ldate.Format(_T("%#m"));
  _tprintf(_T("Month without leading zeroes: %s\n"), s.c_str());
  s = ldate.Format(_T("%#U"));
  _tprintf(_T("Week of year without leading zeroes, with Sunday as first day of week: %s\n"), s.c_str());
  s = ldate.Format(_T("%#y"));
  _tprintf(_T("Year without century without leading zeroes: %s\n"), s.c_str());
  s = ldate.Format(_T("%H"));
  _tprintf(_T("Hours in the current day: %s\n"), s.c_str());
  s = ldate.Format(_T("%h"));
  _tprintf(_T("AM / PM Hours in the current day: %s\n"), s.c_str());
#ifdef _WIN32
  s = ldate.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the current day: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = ldate.Format(_T("%M"));
  _tprintf(_T("Minutes in the current day: %s\n"), s.c_str());
  s = ldate.Format(_T("%S"));
  _tprintf(_T("Seconds in the current day: %s\n"), s.c_str());
  s = ldate.Format(_T("%#H"));
  _tprintf(_T("Hours in the current day without leading zeros: %s\n"), s.c_str());
  s = ldate.Format(_T("%#h"));
  _tprintf(_T("AM / PM Hours in the current day without leading zeros: %s\n"), s.c_str());
  s = ldate.Format(_T("%#M"));
  _tprintf(_T("Minutes in the current day without leading zeros: %s\n"), s.c_str());
  s = ldate.Format(_T("%#S"));
  _tprintf(_T("Seconds in the current day without leading zeros: %s\n"), s.c_str());
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  s = ldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("The Current UTC date is %s\n"), s.c_str());

  //Test of the Collate and SetFromCollate functions
  {
    CDate<CGregorianCalendar> k2{CDate<CGregorianCalendar>::CurrentLocalDate()};
    const int32_t c2{k2.Collate()};
    k2.SetFromCollate(c2);
    CLTimeOfDay<> f{CLTimeOfDay<>{1, 2, 3}};
    const uint32_t j{f.Collate()};
    f.SetFromCollate(j);
    const CLDate<CGregorianCalendar> g{CLDate<CGregorianCalendar>::CurrentLocalTime()};
    UNREFERENCED_PARAMETER(g);
  }


  //Test of the CLLTimeSpan class

  //first the constructors
  CLLTimeSpan lts;
  DTIME_ASSERT(!lts.IsValid());

  lts = CLLTimeSpan{3, 4, 5, 6.23456789};
  dd = lts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = lts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = lts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  const double lss{lts.GetSeconds()};
  UNREFERENCED_PARAMETER(lss);
  s = lts.Format(_T("%D %H:%M:%S"));
#ifdef _WIN32
  s = lts.Format(CLTimeSpan::GetWindowsFormatString());
#endif //#ifdef _WIN32

  lts = CLLTimeSpan{3, 27, 59, 69.34567890};
  s = lts.Format(_T("%D %H:%M:%S"));

  lts = CLLTimeSpan{-3, 4, 5, 6.34567890123};
  s = lts.Format(_T("%D %H:%M:%S"));

  CLLTimeSpan lts2{lts};
  s = lts2.Format(_T("%D %H:%M:%S"));

#ifdef _AFX
  lts = CLLTimeSpan{OneDay};
  s = lts.Format(_T("%D %H:%M:%S"));
#endif //#ifdef _AFX

  lts = CLLTimeSpan{86400};
  s = lts.Format(_T("%D %H:%M:%S"));

  lts = CLLTimeSpan{-23};
  dd = lts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = lts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = lts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss2 = lts.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = lts.Format(_T("%D %H:%M:%S"));
  secs = lts.GetValue(); //NOLINT(clang-analyzer-deadcode.DeadStores)

  lts = CLLTimeSpan{-86401};
  dd = lts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = lts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = lts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss2 = lts.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = lts.Format(_T("%D %H:%M:%S"));
  secs = lts.GetValue(); //NOLINT(clang-analyzer-deadcode.DeadStores)

  lts = CLLTimeSpan{-86401};
  dd = lts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = lts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = lts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss2 = lts.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = lts.Format(_T("%D %H:%M:%S"));
  secs = lts.GetValue(); //NOLINT(clang-analyzer-deadcode.DeadStores)

  lts = CLLTimeSpan{86399.99999999};
  s = lts.Format(_T("%D %H:%M:%S"));
  secs = lts.GetValue(); //NOLINT(clang-analyzer-deadcode.DeadStores)

#ifdef _AFX
  lts = CLLTimeSpan{olets};
  s = lts.Format(_T("%D %H:%M:%S"));
#endif //#ifdef _AFX

  lts = CLLTimeSpan{3155760000.0};
  dd = lts.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = lts.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = lts.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss2 = lts.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = lts.Format(_T("%D %H:%M:%S"));

  //Set operators have been tested via the constructors

  //static Constructors
  lts = CLLTimeSpan::OneDay();
  s = lts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("One Day as a time span is %s\n"), s.c_str());
  lts = CLLTimeSpan::OneHour();
  s = lts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("One Hour as a time span is %s\n"), s.c_str());
  lts = CLLTimeSpan::OneMinute();
  s = lts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("One Minute as a time span is %s\n"), s.c_str());
  lts = CLLTimeSpan::OneSecond();
  s = lts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("One Second as a time span is %s\n"), s.c_str());

  //days, Hours, Minutes & seconds accessors
  //already tested via Format

  //Overloaded Arithmetic Operators
  CLLTimeSpan lts1{CLLTimeSpan{365, 0, 0, 0}};
  lts2 = CLLTimeSpan::OneDay();
  //operator= already tested
  CLLTimeSpan lts3{lts1 + lts2};
  _tprintf(_T("365 days + One day is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts1 - lts2;
  _tprintf(_T("365 days - One day is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts2 - lts1;

  _tprintf(_T("One day - One year is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts2 = CLLTimeSpan::OneDay();
  lts1 += lts2;
  _tprintf(_T("365 days incremented by one day is %s\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  lts1 -= lts2;
  _tprintf(_T("365 days + one day decremented by one day is %s\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts2 -= lts1;
  _tprintf(_T("One day decremented by 365 days is %s\n"), lts2.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts1.Negate();
  lts2 = CLLTimeSpan::OneDay();
  lts3 = lts1 - lts2;
  _tprintf(_T("-365 days - one day is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts1 + lts2;
  _tprintf(_T("-365 days + one day is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts2 = CLLTimeSpan::OneDay();
  lts2.Negate();
  lts3 = lts1 - lts2;
  _tprintf(_T("365 days - (-one day) is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts1 + lts2;
  _tprintf(_T("365 days + (-one day) is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts1.Negate();
  lts2 = CLLTimeSpan::OneDay();
  lts1 += lts2;
  _tprintf(_T("-365 days incremented by one day is %s\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  lts1 -= lts2;
  _tprintf(_T("-365 days + 1 day decremented by one day is %s\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  lts2 = CLLTimeSpan{365, 0, 0, 0};
  lts2.Negate();
  lts1 -= lts2;
  _tprintf(_T("-365 days decremented by -one year is %s\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  lts1 += lts2;
  _tprintf(_T("-365 days - -365 days incremented by -365 days is %s\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts2 = CLLTimeSpan::OneDay();
  lts2.Negate();
  lts3 = lts1 - lts2;
  _tprintf(_T("365 days - -one day is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts1 + lts2;
  _tprintf(_T("365 days + -one day is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts2 - lts1;
  _tprintf(_T("-One day - 365 days is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts2 + lts1;
  _tprintf(_T("-One day + 365 days is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts2 = CLLTimeSpan::OneDay();
  lts3 = lts1 * 100.0;

  dd = lts3.GetTotalDays(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  hh = lts3.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  mm = lts3.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss2 = lts3.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = lts3.Format(_T("%D %H:%M:%S"));

  _tprintf(_T("365 days * 100 is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = 100 * lts1;
  _tprintf(_T("100 * 365 days is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts1 / 3;
  _tprintf(_T("365 days / 3 is %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts1 *= 100;
  _tprintf(_T("365 days *= 100 is %s\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts1 /= 100;
  _tprintf(_T("365 days /= 100 is %s\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  lts1 = CLLTimeSpan{365, 6, 10, 34};
  lts3 = lts1 * 100;
  _tprintf(_T("%s * 100 is"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  _tprintf(_T(" %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());
  lts3 = lts1 / 100;
  _tprintf(_T("%s / 100 is"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  _tprintf(_T(" %s\n"), lts3.Format(_T("%D %H:%M:%S")).c_str());

  s = lts1.Format(_T("%D %H:%M:%S"));
  lts3 = - lts1;
  s = lts3.Format(_T("%D %H:%M:%S"));

  //overloaded equality operators
  lts1 = CLLTimeSpan{365, 0, 0, 0};
  lts2 = CLLTimeSpan::OneDay();
  DTIME_ASSERT(lts1 >= lts2);
  DTIME_ASSERT(lts1 > lts2);
  DTIME_ASSERT(lts2 < lts1);
  DTIME_ASSERT(lts2 <= lts1);
  DTIME_ASSERT(lts2 != lts1);
  DTIME_ASSERT(!(lts2 == lts1));

  //Full test of the Format function
  lts1 = CLLTimeSpan{1, 2, 3, 4.5678901};
  _tprintf(_T("A full test of CLLTimeSpan::Format for %s gives\n"), lts1.Format(_T("%D %H:%M:%S")).c_str());
  s = lts1.Format(_T("%D"));
  _tprintf(_T("Days in the timespan: %s\n"), s.c_str());
  s = lts1.Format(_T("%H"));
  _tprintf(_T("Hours in the timespan: %s\n"), s.c_str());
  s = lts1.Format(_T("%M"));
  _tprintf(_T("Minutes in the timespan: %s\n"), s.c_str());
  s = lts1.Format(_T("%S"));
  _tprintf(_T("Seconds in the timespan: %s\n"), s.c_str());
  s = lts1.Format(_T("%#H"));
  _tprintf(_T("Hours in the timespan without leading zeros: %s\n"), s.c_str());
  s = lts1.Format(_T("%#M"));
  _tprintf(_T("Minutes in the timespan without leading zeros: %s\n"), s.c_str());
  s = lts1.Format(_T("%#S"));
  _tprintf(_T("Seconds in the timespan without leading zeros: %s\n"), s.c_str());


  //Try out of the CLLTimeOfDay class

  //first the constructors
  CLLTimeOfDay<> ltod;
  DTIME_ASSERT(!ltod.IsValid());

  //ltod = CLLTimeOfDay<>{23, 59, 61, true}; //should assert
  //ltod = CLLTimeOfDay<>{24, 59, 61, true}; //should assert
  //ltod = CLLTimeOfDay<>{23, 59, 69, true}; //should assert

#ifdef _WIN32
  GetLocalTime(&stnow);
  ltod = CLLTimeOfDay<>{stnow};
  s = ltod.Format(_T("%H:%M:%S"));
#endif //#ifdef _WIN32

  CLLTimeOfDay<> ltod2{ltod};
  s = ltod.Format(_T("%H:%M:%S"));

  //ltod2 = CLLTimeOfDay<>{-23.5}; //should assert
  //s = ltod2.Format(_T("%H:%M:%S"));

  //ltod2 = CLLTimeOfDay<>{-86401.5}; //should assert
  //s = ltod2.Format(_T("%H:%M:%S"));

  ltod2 = CLLTimeOfDay<>{86399.999};
  s = ltod2.Format(_T("%H:%M:%S"));

  ltod2 = CLLTimeOfDay<>{86199.9766};
  s = ltod2.Format(_T("%H:%M:%S"));

  //Set functions tested via the constructors

  //The static constructors
  ltod = CLLTimeOfDay<>::CurrentLocalTime();
  s = ltod.Format(_T("%H:%M:%S"));
  _tprintf(_T("The local time of day is %s\n"), s.c_str());

  ltod = CLLTimeOfDay<>::CurrentSystemTime();
  s = ltod.Format(_T("%H:%M:%S"));
  _tprintf(_T("The UTC time of day is %s\n"), s.c_str());

  ltod = CLLTimeOfDay<>::Midnight();
  s = ltod.Format(_T("%H:%M:%S"));
  _tprintf(_T("Midnight as a time of day is %s\n"), s.c_str());

  ltod = CLLTimeOfDay<>::Midday();
  s = ltod.Format(_T("%H:%M:%S"));
  _tprintf(_T("Midday as a time of day is %s\n"), s.c_str());

  //try out the operations
  ltod = CLLTimeOfDay<>::CurrentSystemTime();
  wVal = ltod.GetHours(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  wVal = ltod.GetMinutes(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss2 = ltod.GetTotalSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  ss2 = ltod.GetSeconds(); //NOLINT(clang-analyzer-deadcode.DeadStores)
  DTIME_ASSERT(ltod.IsValid());
#ifdef _WIN32
  bSuccess = ltod.GetAsSystemTime(stnow); //NOLINT(clang-analyzer-deadcode.DeadStores)
  s = ltod.Format(CLTimeOfDay<>::GetWindowsFormatString());
#endif //#ifdef _WIN32

  //Arithmetic operators
  CLLTimeOfDay<> ltod3{12, 1, 2.3456};
  CLLTimeSpan  lts4{1, 25, 59, 45.67};
  //operator= already tested
  CLLTimeOfDay<> ltod4{ltod3 + lts4};
  s = ltod4.Format(_T("%H:%M:%S"));
  s2 = ltod3.Format(_T("%H:%M:%S"));
  s3 = lts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  _tprintf(_T(" + %s = "), s3.c_str());
  _tprintf(_T("%s\n"), s.c_str());

  ltod2 = ltod3 - lts4;
  s2 = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  s3 = lts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" - %s = "), s3.c_str());
  s = ltod2.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  s2 = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  ltod3 += lts4;
  s3 = lts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" += %s = "), s3.c_str());
  s = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  s2 = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  ltod3 -= lts4;
  s3 = lts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" -= %s = "), s3.c_str());
  s = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  //try out negative timespans
  lts4.Negate();
  ltod4 = ltod3 + lts4;
  s2 = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  s3 = lts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" + %s = "), s3.c_str());
  s = ltod4.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  ltod2 = ltod3 - lts4;
  s2 = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  s3 = lts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" - %s = "), s3.c_str());
  s = ltod2.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  s2 = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  ltod3 += lts4;
  s3 = lts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" += %s = "), s3.c_str());
  s = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  s2 = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s"), s2.c_str());
  ltod3 -= lts4;
  s3 = lts4.Format(_T("%D %H:%M:%S"));
  _tprintf(_T(" -= %s = "), s3.c_str());
  s = ltod3.Format(_T("%H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  //equality operators

  CLLTimeOfDay<> ltod1{12, 1, 2};
  ltod2 = CLLTimeOfDay<>{13, 0, 2};

  DTIME_ASSERT(ltod1 <= ltod2);
  DTIME_ASSERT(ltod1 < ltod2);
  DTIME_ASSERT(ltod2 > ltod1);
  DTIME_ASSERT(ltod2 >= ltod1);
  DTIME_ASSERT(ltod2 != ltod1);
  DTIME_ASSERT(!(ltod2 == ltod1));

  //Full test of the Format function
  ltod1 = CLLTimeOfDay<>::CurrentLocalTime();
  s = ltod1.Format(_T("%H:%M:%S"));
  _tprintf(_T("A full test of CLTimeOfDay<>::Format for Now (LOCAL) gives\n"));
  s = ltod1.Format(_T("%H"));
  _tprintf(_T("Hours in the current day: %s\n"), s.c_str());
  s = ltod1.Format(_T("%h"));
  _tprintf(_T("AM / PM Hours in the current day: %s\n"), s.c_str());
#ifdef _WIN32
  s = ltod1.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the current day: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = ltod1.Format(_T("%M"));
  _tprintf(_T("Minutes in the current day: %s\n"), s.c_str());
  s = ltod1.Format(_T("%S"));
  _tprintf(_T("Seconds in the current day: %s\n"), s.c_str());
  s = ltod1.Format(_T("%#H"));
  _tprintf(_T("Hours in the current day without leading zeros: %s\n"), s.c_str());
  s = ltod1.Format(_T("%#h"));
  _tprintf(_T("AM / PM Hours in the current day without leading zeros: %s\n"), s.c_str());
  s = ltod1.Format(_T("%#M"));
  _tprintf(_T("Minutes in the current day without leading zeros: %s\n"), s.c_str());
  s = ltod1.Format(_T("%#S"));
  _tprintf(_T("Seconds in the current day without leading zeros: %s\n"), s.c_str());
  ltod2 = CLLTimeOfDay<>{11, 1, 2};
#ifdef _WIN32
  s = ltod2.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the 11:01:02: %s\n"), s.c_str());
  ltod2 = CLLTimeOfDay<>{12, 1, 2};
  s = ltod2.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the 12:01:02: %s\n"), s.c_str());
  ltod2 = CLLTimeOfDay<>{13, 1, 2};
  s = ltod2.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the 13:01:02: %s\n"), s.c_str());
#endif //#ifdef _WIN32


  //Try out the CLLDate class
  _tprintf(_T("Doing CLLDate Gregorian calendar validation Phase 1 (Using 00:00:00 time of day)\n"));
  StartYear = -2000000;
  EndYear = 2000000;
  double fLastLLDateValue{0};
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  bLastDateValueValid = false;
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(Month::January); wMonth<=static_cast<uint16_t>(Month::December); wMonth++)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        const CLLDate<CGregorianCalendar> lldate{lYear, wMonth, wDay, 0, 0, 0};
        if (ldate.IsValid())
        {
          const DateLLS lds{lldate.GetDate()};
          if ((lds.Year != lYear) || (lds.Month != wMonth) || (lds.Day != wDay) || (lds.Hour != 0) || (lds.Minute != 0) || (lds.Second != 0))
          {
            _tprintf(_T("Error occurred testing %d/%d/%d 00:00:00\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (fabs(lldate.GetValue() - (fLastLLDateValue + 86400)) > 0.0001)
            {
              _tprintf(_T("Error occurred testing %d/%d/%d 00:00:00\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              DTIME_ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          fLastLLDateValue = lldate.GetValue();
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 1 time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, static_cast<uint16_t>(Month::December), CGregorianCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(Month::December))} - CDate<CGregorianCalendar>{StartYear, static_cast<uint16_t>(Month::January), 1};
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 1 days roundtripped: %d\n"), nDays);
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 1 speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CLLDate Gregorian calendar validation Phase 2 (Using 01:02:03.456 time of day)\n"));
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(Month::January); wMonth<=static_cast<uint16_t>(Month::December); wMonth++)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        const CLLDate<CGregorianCalendar> lldate{lYear, wMonth, wDay, 1, 2, 3.456};
        const double fValue{lldate.GetValue()};
        const double fLower{nextafter(fValue, fValue - 1.0)};
        const double fUpper{nextafter(fValue, fValue + 1.0)};
        const CLLDate<CGregorianCalendar> lllower{fLower};
        const CLLDate<CGregorianCalendar> llupper{fUpper};
        if (lldate.IsValid())
        {
          bool bValid{false};
          const DateLLS llds{lldate.GetDate()};
          if ((llds.Year != lYear) || (llds.Month != wMonth) || (llds.Day != wDay))
          {
            const double fLowerTOD{lllower.GetCLLTimeOfDay().GetTotalSeconds()};
            const double fTOD{lldate.GetCLLTimeOfDay().GetTotalSeconds()};
            bValid = (fabs(fTOD + 86400.0 - fLowerTOD) < 0.008); //We must have just wrapped around midnight. 0.008 is about the worst case resolution of a double for the test range of the years -2000000 to 2000000
          }
          else
          {
            const double fLowerTOD{lllower.GetCLLTimeOfDay().GetTotalSeconds()};
            const double fTOD{lldate.GetCLLTimeOfDay().GetTotalSeconds()};
            bValid = (fabs(fTOD - fLowerTOD) < 0.008); //0.008 is about the worst case resolution of a double for the test range of the years -2000000 to 2000000
          }
          if (!bValid)
          {
            _tprintf(_T("Error occurred testing %d/%d/%d 01:02:03.456\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 2 time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, static_cast<uint16_t>(Month::December), CGregorianCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(Month::December))} - CDate<CGregorianCalendar>{StartYear, static_cast<uint16_t>(Month::January), 1};
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 2 days roundtripped: %d\n"), nDays);
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 2 speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CLLDate Gregorian calendar validation Phase 3 (Using 23:59:59.9965 time of day)\n"));
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(Month::January); wMonth<=static_cast<uint16_t>(Month::December); wMonth++)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
        const CLLDate<CGregorianCalendar> lldate{lYear, wMonth, wDay, 23, 59, 59.9965};
        const double fValue{lldate.GetValue()};
        const double fLower{nextafter(fValue, fValue - 1.0)};
        const CLLDate<CGregorianCalendar> lllower{fLower};
        if (lldate.IsValid())
        {
          bool bValid{false};
          const DateLLS llds{lldate.GetDate()};
          if ((llds.Year != lYear) || (llds.Month != wMonth) || (llds.Day != wDay))
          {
            const double fLowerTOD{lllower.GetCLLTimeOfDay().GetTotalSeconds()};
            const double fTOD{lldate.GetCLLTimeOfDay().GetTotalSeconds()};
            bValid = (fabs(fTOD + 86400.0 - fLowerTOD) < 0.008); //We must have just wrapped around midnight. 0.008 is about the worst case resolution of a double for the test range of the years -2000000 to 2000000
          }
          else
          {
            const double fLowerTOD{lllower.GetCLLTimeOfDay().GetTotalSeconds()};
            const double fTOD{lldate.GetCLLTimeOfDay().GetTotalSeconds()};
            bValid = (fabs(fTOD - fLowerTOD) < 0.008); //0.008 is about the worst case resolution of a double for the test range of the years -2000000 to 2000000
          }
          if (!bValid)
          {
            _tprintf(_T("Error occurred testing %d/%d/%d 23:59:59.9965\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            DTIME_ASSERT(false);
          }
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 3 time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, static_cast<uint16_t>(Month::December), CGregorianCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(Month::December))} - CDate<CGregorianCalendar>{StartYear, static_cast<uint16_t>(Month::January), 1};
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 3 days roundtripped: %d\n"), nDays);
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 3 speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  _tprintf(_T("Doing CLLDate Gregorian calendar validation Phase 4 (Using random time of day)\n"));
  startTime = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  for (lYear=StartYear; lYear<EndYear; lYear++)
  {
    if ((lYear % 50000) == 0)
      _tprintf(_T("Testing year %d\n"), lYear);
    for (wMonth=static_cast<uint16_t>(Month::January); wMonth<=static_cast<uint16_t>(Month::December); wMonth++)
    {
      const uint16_t wLastDayInMonth{CGregorianCalendar::DaysInMonth(lYear, wMonth)};
      for (wDay=1; wDay<=wLastDayInMonth; wDay++)
      {
#ifdef _MSC_VER
#pragma warning(suppress: 26472)
#endif //#ifdef _MSC_VER
        const auto nRandomHour{static_cast<uint16_t>(rand() % 24)};
#ifdef _MSC_VER
#pragma warning(suppress: 26472)
#endif //#ifdef _MSC_VER
        const auto nRandomMinute{static_cast<uint16_t>(rand() % 60)};
#ifdef _MSC_VER
#pragma warning(suppress: 26472)
#endif //#ifdef _MSC_VER
        const auto nRandomSecond{static_cast<uint16_t>(rand() % 60)};
        const double fRandomSecond{nRandomSecond + (static_cast<double>(rand()) / (RAND_MAX + 1.0))};
        const CLLDate<CGregorianCalendar> lldate{lYear, wMonth, wDay, nRandomHour, nRandomMinute, fRandomSecond};
        const double fValue{lldate.GetValue()};
        const double fLower{nextafter(fValue, fValue - 1.0)};
        const CLLDate<CGregorianCalendar> lllower{fLower};
        if (lldate.IsValid())
        {
          bool bValid{false};
          const DateLLS llds{lldate.GetDate()};
          if ((llds.Year != lYear) || (llds.Month != wMonth) || (llds.Day != wDay))
          {
            const double fLowerTOD{lllower.GetCLLTimeOfDay().GetTotalSeconds()};
            const double fTOD{lldate.GetCLLTimeOfDay().GetTotalSeconds()};
            bValid = (fabs(fTOD + 86400.0 - fLowerTOD) < 0.008); //We must have just wrapped around midnight. 0.008 is about the worst case resolution of a double for the test range of the years -2000000 to 2000000
          }
          else
          {
            double fLowerTOD{lllower.GetCLLTimeOfDay().GetTotalSeconds()};
            const double fTOD{lldate.GetCLLTimeOfDay().GetTotalSeconds()};
            if ((fTOD < 0.008) && (fLowerTOD > 86399.992))
              fLowerTOD -= 86400.0;
            bValid = (fabs(fTOD - fLowerTOD) < 0.008);
          }
          if (!bValid)
          {
            _tprintf(_T("Error occurred testing %d/%d/%d %02d:%02d:%f\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay), static_cast<int>(nRandomHour), static_cast<int>(nRandomMinute), fRandomSecond);
            DTIME_ASSERT(false);
          }
        }
        else
          DTIME_ASSERT(false);
      }
    }
  }
  fSecondsTaken = (CLLDate<CGregorianCalendar>::CurrentSystemTime() - startTime).GetValue();
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 4 time taken: %f seconds\n"), fSecondsTaken);
  nDays = CDate<CGregorianCalendar>{EndYear - 1, static_cast<uint16_t>(Month::December), CGregorianCalendar::DaysInMonth(EndYear - 1, static_cast<uint16_t>(Month::December))} - CDate<CGregorianCalendar>{StartYear, static_cast<uint16_t>(Month::January), 1};
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 4 days roundtripped: %d\n"), nDays);
  _tprintf(_T("CLLDate Gregorian calendar validation Phase 4 speed: %f nanoseconds / day\n"), fSecondsTaken / nDays * 1000000000);

  //first the constructors
  CLLDate<CGregorianCalendar> xllneg{3, static_cast<uint16_t>(Month::January), 2, 0, 0, 0};
  ld = xllneg.GetCDate();
  s = ld.Format(_T("%Y-%m-%d"));
  const CLLTimeOfDay<> lltod5{xllneg.GetCLLTimeOfDay()};
  UNREFERENCED_PARAMETER(lltod5);
  sneg = xllneg.Format(_T("%Y-%m-%d %H:%M:%S"));
  xllneg = CLLDate<CGregorianCalendar>{3, static_cast<uint16_t>(Month::January), 2, 0, 0, 0};
  sneg = xllneg.Format(_T("%Y-%m-%d %H:%M:%S"));
  xllneg = CLLDate<CGregorianCalendar>{3, static_cast<uint16_t>(Month::January), 2, 4, 3, 2.3456};
  sneg = xllneg.Format(_T("%Y-%m-%d %H:%M:%S"));
  xllneg = CLLDate<CGregorianCalendar>{3, static_cast<uint16_t>(Month::January), 2, 23, 59, 59.8765};
  sneg = xllneg.Format(_T("%Y-%m-%d %H:%M:%S"));

  //values around the epoch
  CLLDate<CGregorianCalendar> lldate6{1582, static_cast<uint16_t>(Month::October), 13, 0, 0, 0};
  DateLLS lds6{lldate6.GetDate()};
  String sneg2{lldate6.Format(_T("%Y-%m-%d %H:%M:%S"))};
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 13, 0, 0, 0.0001};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 13, 23, 59, 59.9999};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 14, 0, 0, 0};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 14, 0, 0, 0.0001};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 14, 23, 59, 59.9999};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 15, 0, 0, 0};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 15, 0, 0, 0.0001};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 15, 23, 59, 59.999};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate6 = CLLDate<CGregorianCalendar>{1582, static_cast<uint16_t>(Month::October), 16, 0, 0, 0};
  lds6 = lldate6.GetDate();
  sneg2 = lldate6.Format(_T("%Y-%m-%d %H:%M:%S"));


  CLLDate<CGregorianCalendar> lldate;
  DTIME_ASSERT(!lldate.IsValid());
  lldate = CLLDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::November), 3, 12, 11, 10.12345};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
#ifdef _WIN32
  GetLocalTime(&st);
  lldate = CLLDate<CGregorianCalendar>{st};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  s = lldate.Format(CLLDate<>::GetWindowsFormatString());
#endif //#ifdef _WIN32
  lldate = CLLDate<CGregorianCalendar>{1995, static_cast<uint16_t>(Month::March), 5, DayOfWeek::Sunday, 0, 0, 0};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  CLLDate<CJulianCalendar> lldate2{CLLDate<CJulianCalendar>{1000, DateEpoch::EPOCH_JD, 0, 0, 0}};
  s = lldate2.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate2 = CLLDate<CJulianCalendar>{-1, DateEpoch::EPOCH_JD, 0, 0, 0};
  s = lldate2.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate = CLLDate<CGregorianCalendar>{1000, DateEpoch::EPOCH_JD, 0, 0, 0};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate = CLLDate<CGregorianCalendar>{-1, DateEpoch::EPOCH_JD, 0, 0, 0};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate = CLLDate<CGregorianCalendar>{1000.0, DateEpoch::EPOCH_JD};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate = CLLDate<CGregorianCalendar>{-1.0, DateEpoch::EPOCH_JD};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate = CLLDate<CGregorianCalendar>{0.5, DateEpoch::EPOCH_CTIME};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate = CLLDate<CGregorianCalendar>{-0.5, DateEpoch::EPOCH_CTIME};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));

  CLLDate<CGregorianCalendar> lldate5{1580, 1, 1, 1, 0, 0.123};
  s = lldate5.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate5.AddYear(1);
  s = lldate5.Format(_T("%Y-%m-%d %H:%M:%S"));

  //Copy constructor already tested
#ifdef _AFX
  ct = CTime::GetCurrentTime();
  lldate = CLLDate<CGregorianCalendar>{ct};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate = CLLDate<CGregorianCalendar>{CDate<CGregorianCalendar>::CurrentLocalDate(), CLLTimeOfDay<>::CurrentLocalTime()};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
#endif //#ifdef _AFX

#ifdef _WIN32
  //Construction using the Win32 FILETIME structure
  GetLocalTime(&st);
  SystemTimeToFileTime(&st, &ft);
  lldate = CLLDate<CGregorianCalendar>{ft};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
#endif //#ifdef _WIN32

#ifdef _AFX
  lldate = CLLDate<CGregorianCalendar>{ldt};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  ldt = COleDateTime{1970, static_cast<int>(Month::February), 1, 5, 6, 7};
  lldate = CLLDate<CGregorianCalendar>{ldt};
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
#endif //#ifdef _AFX

  //Set operators tested via the constructors above

  //Static Constructors
  lldate = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));

  //Need to try out the following functions
  //GetCDate & GetCLTimeOfDay tested through the Format function
#ifdef _WIN32
  bSuccess = lldate.GetAsSystemTime(st); //NOLINT(clang-analyzer-deadcode.DeadStores)
  bSuccess = lldate.GetAsFileTime(ft); //NOLINT(clang-analyzer-deadcode.DeadStores)
#endif //#ifdef _WIN32
#ifdef _AFX
  bSuccess = lldate.GetAsCOleDateTime(dt); //NOLINT(clang-analyzer-deadcode.DeadStores)
#endif //#ifdef _AFX

  lldate.AddYear(10);
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("10 years from now UTC is %s\n"), s.c_str());

  lldate = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  lldate.AddWeek(-10);
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("10 weeks before today UTC is %s\n"), s.c_str());

  lldate = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  lldate.AddMonth(-14);
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("14 months before today UTC is %s\n"), s.c_str());

  DTIME_ASSERT(lldate.IsValid());
  lldate = CLLDate<CGregorianCalendar>{};
  DTIME_ASSERT(!lldate.IsValid());

  lldate = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("The current UTC date is %s\n"), s.c_str());
  lldate = CLLDate<CGregorianCalendar>::CurrentLocalTime();
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("The current local date is %s\n"), s.c_str());

  //operator= already tried out

  lldate = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  const CLLTimeSpan llts{CLLTimeSpan(10, 10, 10, 10.2)};
  lldate += llts;
  s = llts.Format(_T("%D %H:%M:%S"));
  _tprintf(_T("%s from today UTC is "), s.c_str());
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("%s\n"), s.c_str());

  //Arithmetic operators
  lldate = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  CLLDate<CGregorianCalendar> lldate3{lldate};
  s = lldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate3 += CLLTimeSpan{0, 1, 0, 0};
  s = lldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate3 -= CLLTimeSpan{0, 1, 0, 0};
  s = lldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  lts = lldate3 - lldate;
  s = lldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  ++lldate3;
  s = lldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  --lldate3;
  s = lldate3.Format(_T("%Y-%m-%d %H:%M:%S"));

  //try out the equality operators
  lldate = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  CLLDate<CGregorianCalendar> lldate4{lldate};
  DTIME_ASSERT(lldate == lldate4);
  DTIME_ASSERT(!(lldate != lldate4));
  lldate4 += CLLTimeSpan{0, 0, 0, 1.2345};
  DTIME_ASSERT(lldate4 != lldate);
  DTIME_ASSERT(lldate4 >= lldate);
  DTIME_ASSERT(lldate <= lldate4);
  DTIME_ASSERT(lldate < lldate4);

  lldate = CLLDate<CGregorianCalendar>::CurrentLocalTime();
  sString = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate3 = lldate.AsGregorianCLLDate();
  s = lldate3.Format(_T("%Y-%m-%d %H:%M:%S"));
  lldate2 = lldate.AsJulianCLLDate();
  s = lldate2.Format(_T("%Y-%m-%d %H:%M:%S"));

  //Full test of the Format function
  lldate = CLLDate<CGregorianCalendar>::CurrentLocalTime();
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("A full test of CLLDate::Format for Now (LOCAL) gives\n"));
#ifdef _WIN32
  s = lldate.Format(_T("%a"));
  _tprintf(_T("Abbreviated weekday name: %s\n"), s.c_str());
  s = lldate.Format(_T("%A"));
  _tprintf(_T("Full weekday name: %s\n"), s.c_str());
  s = lldate.Format(_T("%b"));
  _tprintf(_T("Abbreviated month name: %s\n"), s.c_str());
  s = lldate.Format(_T("%B"));
  _tprintf(_T("Full month name: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = lldate.Format(_T("%d"));
  _tprintf(_T("Day of month as decimal number (01 - 31): %s\n"), s.c_str());
  s = lldate.Format(_T("%j"));
  _tprintf(_T("Day of year as decimal number (001 - 366): %s\n"), s.c_str());
  s = lldate.Format(_T("%m"));
  _tprintf(_T("Month as decimal number (01 - 12): %s\n"), s.c_str());
  s = lldate.Format(_T("%U"));
  _tprintf(_T("Week of year as decimal number: %s\n"), s.c_str());
  s = lldate.Format(_T("%w"));
  _tprintf(_T("Weekday as decimal number (1 - 7), Sunday is 1: %s\n"), s.c_str());
#ifdef _WIN32
  s = lldate.Format(_T("%x"));
  _tprintf(_T("Date representation for current locale: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = lldate.Format(_T("%y"));
  _tprintf(_T("Year without century, as decimal number (00 - 99): %s\n"), s.c_str());
  s = lldate.Format(_T("%Y"));
  _tprintf(_T("Year with century, as decimal number: %s\n"), s.c_str());
#ifdef _WIN32
  s = lldate.Format(_T("%#x"));
  _tprintf(_T("Long date representation, appropriate to current locale: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = lldate.Format(_T("%#d"));
  _tprintf(_T("Day of month without leading zeros: %s\n"), s.c_str());
  s = lldate.Format(_T("%#j"));
  _tprintf(_T("Day of year without leading zeros: %s\n"), s.c_str());
  s = lldate.Format(_T("%#m"));
  _tprintf(_T("Month without leading zeroes: %s\n"), s.c_str());
  s = lldate.Format(_T("%#U"));
  _tprintf(_T("Week of year without leading zeroes, with Sunday as first day of week: %s\n"), s.c_str());
  s = lldate.Format(_T("%#y"));
  _tprintf(_T("Year without century without leading zeroes: %s\n"), s.c_str());
  s = lldate.Format(_T("%H"));
  _tprintf(_T("Hours in the current day: %s\n"), s.c_str());
  s = lldate.Format(_T("%h"));
  _tprintf(_T("AM / PM Hours in the current day: %s\n"), s.c_str());
#ifdef _WIN32
  s = lldate.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the current day: %s\n"), s.c_str());
#endif //#ifdef _WIN32
  s = lldate.Format(_T("%M"));
  _tprintf(_T("Minutes in the current day: %s\n"), s.c_str());
  s = lldate.Format(_T("%S"));
  _tprintf(_T("Seconds in the current day: %s\n"), s.c_str());
  s = lldate.Format(_T("%#H"));
  _tprintf(_T("Hours in the current day without leading zeros: %s\n"), s.c_str());
  s = lldate.Format(_T("%#h"));
  _tprintf(_T("AM / PM Hours in the current day without leading zeros: %s\n"), s.c_str());
  s = lldate.Format(_T("%#M"));
  _tprintf(_T("Minutes in the current day without leading zeros: %s\n"), s.c_str());
  s = lldate.Format(_T("%#S"));
  _tprintf(_T("Seconds in the current day without leading zeros: %s\n"), s.c_str());
  lldate = CLLDate<CGregorianCalendar>::CurrentSystemTime();
  s = lldate.Format(_T("%Y-%m-%d %H:%M:%S"));
  _tprintf(_T("The Current UTC date is %s\n"), s.c_str());


#ifdef _AFX
  //Get the TEMP path
  CString sTempPath;
  const DWORD dwGTP{GetTempPath(_MAX_PATH, sTempPath.GetBuffer(_MAX_PATH))};
  sTempPath.ReleaseBuffer();
  if (dwGTP == 0)
  {
    _tprintf(_T("Unable to get the TEMP path, Error:%u"), GetLastError()); //NOLINT(clang-diagnostic-format)
    return 1;
  }

  //Generate a temporary file name for testing the archiving
  CString sTempFile;
  const DWORD dwGTFN{GetTempFileName(sTempPath, _T("DTP"), 0, sTempFile.GetBuffer(_MAX_PATH))};
  sTempFile.ReleaseBuffer();
  if (dwGTFN == 0)
  {
    _tprintf(_T("Unable to generate temp filename, Error:%u"), GetLastError()); //NOLINT(clang-diagnostic-format)
    return 1;
  }

  //Display some intro text
  _tprintf(_T("\nThis program is now going to test out the MFC serialization support\n"));
  _tprintf(_T("by creating a file %s and writing from and reading to it.\n"), sTempFile.GetString());
  _tprintf(_T("The file size should end up as 62 bytes\n"));

  //Test out the persistence methods
  CFile datafile;
  CFileException ex;
  if (datafile.Open(sTempFile, CFile::modeWrite | CFile::modeCreate, &ex))
  {
    {
      CArchive archive{&datafile, CArchive::store};

      archive << x;
      _tprintf(_T("Saving %s to archive\n"), x.Format(_T("%Y-%m-%d")).c_str());
      archive << ts;
      _tprintf(_T("Saving %s to archive\n"), ts.Format(_T("%D %H:%M:%S")).c_str());
      archive << tod;
      _tprintf(_T("Saving %s to archive\n"), tod.Format(_T("%H:%M:%S")).c_str());
      archive << ldate;
      _tprintf(_T("Saving %s to archive\n"), ldate.Format(_T("%Y-%m-%d %H:%M:%S")).c_str());
      archive << lts;
      _tprintf(_T("Saving %s to archive\n"), lts.Format(_T("%D %H:%M:%S")).c_str());
      archive << ltod;
      _tprintf(_T("Saving %s to archive\n"), ltod.Format(_T("%H:%M:%S")).c_str());
      archive << lldate;
      _tprintf(_T("Saving %s to archive\n"), lldate.Format(_T("%Y-%m-%d %H:%M:%S")).c_str());
    }
    datafile.Close();
  }
  else
    _tprintf(_T("Unable to open archive file for testing writing, Error:%d\n"), ex.m_lOsError); //NOLINT(clang-diagnostic-format)
  if (datafile.Open(sTempFile, CFile::modeRead, &ex))
  {
    CArchive archive{&datafile, CArchive::load};

    CDate<CGregorianCalendar> xin;
    archive >> xin;
    _tprintf(_T("Loaded %s from archive\n"), xin.Format(_T("%Y-%m-%d")).c_str());
    CLTimeSpan tsin;
    archive >> tsin;
    _tprintf(_T("Loaded %s from archive\n"), tsin.Format(_T("%D %H:%M:%S")).c_str());
    CLTimeOfDay<> todin;
    archive >> todin;
    _tprintf(_T("Loaded %s from archive\n"), todin.Format(_T("%H:%M:%S")).c_str());
    CLDate<CGregorianCalendar> ldatein;
    archive >> ldatein;
    _tprintf(_T("Loaded %s from archive\n"), ldatein.Format(_T("%Y-%m-%d %H:%M:%S")).c_str());
    CLLTimeSpan ltsin;
    archive >> ltsin;
    _tprintf(_T("Loaded %s from archive\n"), ltsin.Format(_T("%D %H:%M:%S")).c_str());
    CLLTimeOfDay<> ltodin;
    archive >> ltodin;
    _tprintf(_T("Loaded %s from archive\n"), ltodin.Format(_T("%H:%M:%S")).c_str());
    CLLDate<CGregorianCalendar> lldatein;
    archive >> lldatein;
    _tprintf(_T("Loaded %s from archive\n"), lldatein.Format(_T("%Y-%m-%d %H:%M:%S")).c_str());
  }
  else
    _tprintf(_T("Unable to open archive file for testing reading, Error:%d\n"), ex.m_lOsError); //NOLINT(clang-diagnostic-format)
#endif //#ifdef _AFX

  _tprintf(_T("\nTotal Time taken to execute program in seconds was: %f\n"), (CLDate<CGregorianCalendar>::CurrentSystemTime() - startDate).SecondsAsDouble());

  return 0;
}
